@@ -374,10 +374,8 @@ namespace Microsoft.Build.BuildEngine {
 				return false;
 
 			ITaskItem[] outputs;
-			if (ParentEngine.BuiltTargetsOutputByName.TryGetValue (key, out outputs)) {
-				if (targetOutputs != null)
-					targetOutputs.Add (target_name, outputs);
-			}
+			if (targetOutputs != null && ParentEngine.BuiltTargetsOutputByName.TryGetValue (key, out outputs))
+				targetOutputs [target_name] = outputs;
 			return true;
 		}
 
@@ -1077,6 +1075,11 @@ namespace Microsoft.Build.BuildEngine {
 			if (!String.IsNullOrEmpty (ToolsVersion))
 				return ToolsVersion;
 
+#if XBUILD_14
+			return "14.0";
+#elif XBUILD_12
+			return "12.0";
+#else
 			if (!HasToolsVersionAttribute)
 				return parentEngine.DefaultToolsVersion;
 
@@ -1088,6 +1091,7 @@ namespace Microsoft.Build.BuildEngine {
 			}
 
 			return DefaultToolsVersion;
+#endif
 		}
 		
 		void AddProjectExtensions (XmlElement xmlElement)
