@@ -117,6 +117,8 @@ namespace Mono.Debugger.Soft
 		public string[] names;
 		public int[] live_range_start;
 		public int[] live_range_end;
+		public int[] scopes_start;
+		public int[] scopes_end;
 	}
 
 	struct PropInfo {
@@ -155,11 +157,12 @@ namespace Mono.Debugger.Soft
 
 	[Flags]
 	enum InvokeFlags {
-		NONE = 0x0,
-		DISABLE_BREAKPOINTS = 0x1,
-		SINGLE_THREADED = 0x2,
-		OUT_THIS = 0x4,
-		OUT_ARGS = 0x8,
+		NONE = 0,
+		DISABLE_BREAKPOINTS = 1,
+		SINGLE_THREADED = 2,
+		OUT_THIS = 4,
+		OUT_ARGS = 8,
+		VIRTUAL = 16,
 	}
 
 	enum ElementType {
@@ -381,7 +384,8 @@ namespace Mono.Debugger.Soft
 		ERR_UNLOADED = 103,
 		ERR_NO_INVOCATION = 104,
 		ABSENT_INFORMATION = 105,
-		NO_SEQ_POINT_AT_IL_OFFSET = 106
+		NO_SEQ_POINT_AT_IL_OFFSET = 106,
+		INVOKE_ABORTED = 107
 	}
 
 	public class ErrorHandlerEventArgs : EventArgs {
@@ -416,7 +420,7 @@ namespace Mono.Debugger.Soft
 		 * with newer runtimes, and vice versa.
 		 */
 		internal const int MAJOR_VERSION = 2;
-		internal const int MINOR_VERSION = 35;
+		internal const int MINOR_VERSION = 45;
 
 		enum WPSuspendPolicy {
 			NONE = 0,
@@ -528,7 +532,8 @@ namespace Mono.Debugger.Soft
 			GET_MANIFEST_MODULE = 3,
 			GET_OBJECT = 4,
 			GET_TYPE = 5,
-			GET_NAME = 6
+			GET_NAME = 6,
+			GET_DOMAIN = 7
 		}
 
 		enum CmdModule {
@@ -584,7 +589,9 @@ namespace Mono.Debugger.Soft
 		enum CmdStackFrame {
 			GET_VALUES = 1,
 			GET_THIS = 2,
-			SET_VALUES = 3
+			SET_VALUES = 3,
+			GET_DOMAIN = 4,
+			SET_THIS = 5,
 		}
 
 		enum CmdArrayRef {
@@ -799,6 +806,13 @@ namespace Mono.Debugger.Soft
 				return res;
 			}
 
+			public string ReadUTF16String () {
+				int len = decode_int (packet, ref offset);
+				string res = new String (Encoding.Unicode.GetChars (packet, offset, len));
+				offset += len;
+				return res;
+			}
+
 			public ValueImpl ReadValue () {
 				ElementType etype = (ElementType)ReadByte ();
 
@@ -1214,6 +1228,8 @@ namespace Mono.Debugger.Soft
 
 		bool disconnected;
 
+		internal ManualResetEvent DisconnectedEvent = new ManualResetEvent (false);
+
 		void receiver_thread_main () {
 			while (!closed) {
 				try {
@@ -1230,6 +1246,7 @@ namespace Mono.Debugger.Soft
 
 			lock (reply_packets_monitor) {
 				disconnected = true;
+				DisconnectedEvent.Set ();
 				Monitor.PulseAll (reply_packets_monitor);
 				TransportClose ();
 			}
@@ -1633,13 +1650,14 @@ namespace Mono.Debugger.Soft
 			SendReceive (CommandSet.VM, (int)CmdVM.SET_PROTOCOL_VERSION, new PacketWriter ().WriteInt (major).WriteInt (minor));
 		}
 
-		internal long[] VM_GetThreads () {
-			var res = SendReceive (CommandSet.VM, (int)CmdVM.ALL_THREADS, null);
-			int len = res.ReadInt ();
-			long[] arr = new long [len];
-			for (int i = 0; i < len; ++i)
-				arr [i] = res.ReadId ();
-			return arr;
+		internal void VM_GetThreads (Action<long[]> resultCallaback) {
+			Send (CommandSet.VM, (int)CmdVM.ALL_THREADS, null, (res) => {
+				int len = res.ReadInt ();
+				long[] arr = new long [len];
+				for (int i = 0; i < len; ++i)
+					arr [i] = res.ReadId ();
+				resultCallaback(arr);
+			}, 1);
 		}
 
 		internal void VM_Suspend () {
@@ -1896,6 +1914,19 @@ namespace Mono.Debugger.Soft
 			var res = SendReceive (CommandSet.METHOD, (int)CmdMethod.GET_LOCALS_INFO, new PacketWriter ().WriteId (id));
 
 			LocalsInfo info = new LocalsInfo ();
+
+			if (Version.AtLeast (2, 43)) {
+				int nscopes = res.ReadInt ();
+				info.scopes_start = new int [nscopes];
+				info.scopes_end = new int [nscopes];
+				int last_start = 0;
+				for (int i = 0; i < nscopes; ++i) {
+					info.scopes_start [i] = last_start + res.ReadInt ();
+					info.scopes_end [i] = info.scopes_start [i] + res.ReadInt ();
+					last_start = info.scopes_start [i];
+				}
+			}
+
 			int nlocals = res.ReadInt ();
 			info.types = new long [nlocals];
 			for (int i = 0; i < nlocals; ++i)
@@ -2084,6 +2115,10 @@ namespace Mono.Debugger.Soft
 			return SendReceive (CommandSet.ASSEMBLY, (int)CmdAssembly.GET_NAME, new PacketWriter ().WriteId (id)).ReadString ();
 		}
 
+		internal long Assembly_GetIdDomain (long id) {
+			return SendReceive (CommandSet.ASSEMBLY, (int)CmdAssembly.GET_DOMAIN, new PacketWriter ().WriteId (id)).ReadId ();
+		}
+
 		/*
 		 * TYPE
 		 */
@@ -2377,6 +2412,14 @@ namespace Mono.Debugger.Soft
 			SendReceive (CommandSet.STACK_FRAME, (int)CmdStackFrame.SET_VALUES, new PacketWriter ().WriteId (thread_id).WriteId (id).WriteInt (len).WriteInts (pos).WriteValues (values));
 		}
 
+		internal long StackFrame_GetDomain (long thread_id, long id) {
+			return SendReceive (CommandSet.STACK_FRAME, (int)CmdStackFrame.GET_DOMAIN, new PacketWriter ().WriteId (thread_id).WriteId (id)).ReadId ();
+		}
+
+		internal void StackFrame_SetThis (long thread_id, long id, ValueImpl value) {
+			SendReceive (CommandSet.STACK_FRAME, (int)CmdStackFrame.SET_THIS, new PacketWriter ().WriteId (thread_id).WriteId (id).WriteValue (value));
+		}
+
 		/*
 		 * ARRAYS
 		 */
@@ -2408,7 +2451,16 @@ namespace Mono.Debugger.Soft
 		 * STRINGS
 		 */
 		internal string String_GetValue (long id) {
-			return SendReceive (CommandSet.STRING_REF, (int)CmdStringRef.GET_VALUE, new PacketWriter ().WriteId (id)).ReadString ();
+			var r = SendReceive (CommandSet.STRING_REF, (int)CmdStringRef.GET_VALUE, new PacketWriter ().WriteId (id));
+
+			bool is_utf16 = false;
+			if (Version.AtLeast (2, 41))
+				is_utf16 = r.ReadByte () == 1;
+
+			if (is_utf16)
+				return r.ReadUTF16String ();
+			else
+				return r.ReadString ();
 		}			
 
 		internal int String_GetLength (long id) {
@@ -2469,6 +2521,7 @@ namespace Mono.Debugger.Soft
 		{
 			closed = true;
 			disconnected = true;
+			DisconnectedEvent.Set ();
 			TransportClose ();
 		}
 	}
