@@ -29,6 +29,7 @@
 
 using System;
 using System.Reflection;
+using System.Reflection.Emit;
 using System.Collections.Generic;
 using System.Linq;
 using System.Linq.Expressions;
@@ -87,11 +88,7 @@ namespace MonoTests.System.Linq.Expressions {
 		}
 
 		[Test]
-#if NET_4_0
 		[ExpectedException (typeof (ArgumentException))]
-#else
-		[ExpectedException (typeof (ArgumentNullException))]
-#endif
 		public void ArgInstanceNullForNonStaticMethod ()
 		{
 			Expression.Call (null, typeof (object).GetMethod ("ToString"));
@@ -243,6 +240,17 @@ namespace MonoTests.System.Linq.Expressions {
 			return 42;
 		}
 
+		[Test]
+		public void CallMethodOnDateTime ()
+		{
+			var left = Expression.Call (Expression.Constant (DateTime.Now), typeof(DateTime).GetMethod ("AddDays"), Expression.Constant (-5.0));
+			var right = Expression.Constant (DateTime.Today.AddDays (1));
+			var expr = Expression.GreaterThan (left, right);
+
+			var eq = Expression.Lambda<Func<bool>> (expr).Compile ();
+			Assert.IsFalse (eq ());
+		}
+
 		[Test]
 		[Category ("NotDotNet")] // http://connect.microsoft.com/VisualStudio/feedback/ViewFeedback.aspx?FeedbackID=339351
 		[ExpectedException (typeof (ArgumentException))]
@@ -288,22 +296,42 @@ namespace MonoTests.System.Linq.Expressions {
 			Assert.AreEqual ("foo42", lamda (42, "foo"));
 		}
 
+#if !FULL_AOT_RUNTIME
+		[Test]
+		public void CallDynamicMethod_ToString ()
+		{
+			// Regression test for #49686
+			var m = new DynamicMethod ("intIntId", typeof (int), new Type [] { typeof (int) });
+			var ilg = m.GetILGenerator ();
+			ilg.Emit (OpCodes.Ldarg_0);
+			ilg.Emit (OpCodes.Ret);
+
+			var i = Expression.Parameter (typeof (int), "i");
+			var e = Expression.Call (m, i);
+
+			Assert.IsNotNull (e.ToString ());
+		}
+
+		[Test]
+		public void CallDynamicMethod_CompileInvoke ()
+		{
+			var m = new DynamicMethod ("intIntId", typeof (int), new Type [] { typeof (int) });
+			var ilg = m.GetILGenerator ();
+			ilg.Emit (OpCodes.Ldarg_0);
+			ilg.Emit (OpCodes.Ret);
+
+			var i = Expression.Parameter (typeof (int), "i");
+			var e = Expression.Call (m, i);
+
+			var lambda = Expression.Lambda<Func<int, int>> (e, i).Compile ();
+			Assert.AreEqual (42, lambda (42));
+		}
+#endif
+
 		public static int Bang (Expression i)
 		{
 			return (int) (i as ConstantExpression).Value;
 		}
-#if !NET_4_0 // dlr bug 5875
-		[Test]
-		public void CallMethodWithExpressionParameter ()
-		{
-			var call = Expression.Call (GetType ().GetMethod ("Bang"), Expression.Constant (42));
-			Assert.AreEqual (ExpressionType.Quote, call.Arguments [0].NodeType);
-
-			var l = Expression.Lambda<Func<int>> (call).Compile ();
-
-			Assert.AreEqual (42, l ());
-		}
-#endif
 		static bool fout_called = false;
 
 		public static int FooOut (out int x)
@@ -398,7 +426,6 @@ namespace MonoTests.System.Linq.Expressions {
 		}
 
 		[Test]
-		[Category ("NotWorkingInterpreter")]
 		public void Connect282702 ()
 		{
 			var lambda = Expression.Lambda<Func<Func<int>>> (
