@@ -21,6 +21,8 @@ namespace MonoTests.System.Xml.Xsl
 	[TestFixture]
 	public class XslTransformTests
 	{
+		static readonly string end_of_line = "\r\n";
+
 		XmlDocument doc;
 		XslTransform xslt;
 		XmlDocument result;
@@ -36,10 +38,11 @@ namespace MonoTests.System.Xml.Xsl
 		[Test]
 		public void TestBasicTransform ()
 		{
+			string output = Path.Combine (Path.GetTempPath (), "result.xml");
 			doc.LoadXml ("<root/>");
 			xslt.Load ("Test/XmlFiles/xsl/empty.xsl");
-			xslt.Transform ("Test/XmlFiles/xsl/empty.xsl", "Test/XmlFiles/xsl/result.xml");
-			result.Load ("Test/XmlFiles/xsl/result.xml");
+			xslt.Transform ("Test/XmlFiles/xsl/empty.xsl", output);
+			result.Load (output);
 			Assert.AreEqual (2, result.ChildNodes.Count, "count");
 		}
 
@@ -149,11 +152,6 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 		}
 
 		[Test]
-		[Category ("NotDotNet")]
-		// Actually MS.NET here throws XsltException, but Mono returns
-		// XPathException (since XPath evaluation engine generally
-		// catches (should catch) static error. It is implementation 
-		// dependent matter.
 		[ExpectedException (typeof (XPathException))]
 		public void MSXslNodeSetRejectsNodeSet ()
 		{
@@ -174,6 +172,33 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 			t.Transform (new XPathDocument (new XmlTextReader (new StringReader ("<root><foo attr='A'/><foo attr='B'/><foo attr='C'/></root>"))), null, sw);
 		}
 
+		[Test]
+		[Category ("MobileNotWorking")]
+		public void MSXslFormatDate ()
+		{
+			var arguments = new XsltArgumentList();
+			arguments.AddParam("date", "", new DateTime (2010, 11, 22, 5, 4, 3));
+
+			string xsl = @"
+<xsl:stylesheet version=""1.0"" xmlns=""http://www.w3.org/1999/xhtml"" xmlns:xsl=""http://www.w3.org/1999/XSL/Transform"" xmlns:msxsl=""urn:schemas-microsoft-com:xslt"" exclude-result-prefixes=""msxsl"">
+<xsl:param name='date'/>
+<xsl:template match='/'>
+	<root>
+		<p>The current date is <xsl:value-of select=""msxsl:format-date($date, 'd MMMM yyyy')""/> and current time is <xsl:value-of select=""msxsl:format-time($date, 'HH:mm')""/>.</p>
+	</root>
+</xsl:template>
+</xsl:stylesheet>";
+
+			StringWriter sw = new StringWriter ();
+			var t = new XslCompiledTransform ();
+			t.Load (new XPathDocument (new StringReader (xsl)));
+
+			t.Transform (new XPathDocument (new XmlTextReader (new StringReader ("<root></root>"))), arguments, sw);
+
+			Assert.AreEqual ("<?xml version=\"1.0\" encoding=\"utf-16\"?><root xmlns=\"http://www.w3.org/1999/xhtml\"><p>The current date is 22 November 2010 and current time is 05:04.</p></root>", sw.ToString ());
+		}
+
+
 		[Test]
 		public void EvaluateEmptyVariableAsBoolean ()
 		{
@@ -414,9 +439,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 
 		// http://support.microsoft.com/default.aspx?scid=kb;en-us;834667
 		[Test]
-#if NET_1_1
-		[Category ("NotDotNet")]
-#endif
+		//[Category ("NotDotNet")]
 		public void LocalParameter ()
 		{
 			string xsltFragment = @"<?xml version=""1.0"" encoding=""UTF-8"" ?>
@@ -1000,7 +1023,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"  <something>{0}" +
 				"    <else />{0}" +
 				"  </something>{0}" +
-				"</test>", Environment.NewLine), sw.ToString (), "#1");
+				"</test>", end_of_line), sw.ToString (), "#1");
 
 			// set indent to no
 			sw.GetStringBuilder ().Length = 0;
@@ -1135,7 +1158,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"  <something>{0}" +
 				"    <else />{0}" +
 				"  </something>{0}" +
-				"</test>", Environment.NewLine), sw.ToString (), "#1");
+				"</test>", end_of_line), sw.ToString (), "#1");
 
 			// set indent to no
 			sw.GetStringBuilder ().Length = 0;
@@ -1586,7 +1609,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"{1}<Body>{0}" +
 				"{1}{1}<BR>{0}" +
 				"{1}</Body>{0}" +
-				"</html>", Environment.NewLine, "  "), sw.ToString (), "#1");
+				"</html>", end_of_line, "  "), sw.ToString (), "#1");
 
 			// indent no, media-type not set
 			options = "indent=\"no\"";
@@ -1625,7 +1648,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"{1}<Body>{0}" +
 				"{1}{1}<BR>{0}" +
 				"{1}</Body>{0}" +
-				"</html>", Environment.NewLine, "  "), sw.ToString (), "#3");
+				"</html>", end_of_line, "  "), sw.ToString (), "#3");
 		}
 
 		[Test]
@@ -1675,7 +1698,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"{1}<Body>{0}" +
 				"{1}{1}<BR>{0}" +
 				"{1}</Body>{0}" +
-				"</html>", Environment.NewLine, "  "), result, "#1");
+				"</html>", end_of_line, "  "), result, "#1");
 
 			// indent no, media-type not set
 			options = "indent=\"no\"";
@@ -1722,7 +1745,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"{1}<Body>{0}" +
 				"{1}{1}<BR>{0}" +
 				"{1}</Body>{0}" +
-				"</html>", Environment.NewLine, "  "), result, "#3");
+				"</html>", end_of_line, "  "), result, "#3");
 		}
 
 		[Test]
@@ -1809,7 +1832,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"  <tr>{0}" +
 				"    <td>1,000</td>{0}" +
 				"  </tr>{0}" +
-				"</table>", Environment.NewLine), sw.ToString ());
+				"</table>", end_of_line), sw.ToString ());
 		}
 
 		// http://support.microsoft.com/kb/293469/en-us
@@ -1846,7 +1869,7 @@ xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:msxsl='urn:schemas-micros
 				"<?xml version=\"1.0\" encoding=\"utf-16\"?>{0}" +
 				"<html>{0}" +
 				"  <body>44.442,70</body>{0}" +
-				"</html>", Environment.NewLine), sw.ToString ());
+				"</html>", end_of_line), sw.ToString ());
 		}
 
 		[Test]
@@ -2227,7 +2250,6 @@ NO
 			Assert.IsTrue (sw.ToString ().IndexOf ("NO") > 0);
 		}
 
-#if NET_2_0
 		[Test] // bug #349375
 		public void PreserveWhitespace ()
 		{
@@ -2332,7 +2354,7 @@ NO
   </l0>
 </root>")), null, sw);
 //			Assert.AreEqual (@"<y><t yes-one-node="""">1</t><t not-node=""""></t></y>", sw.ToString ());
-			Assert.AreEqual ("\r\n  \r\n    <y><t yes-one-node=\"\">1</t><t not-node=\"\"></t></y>\r\n  \r\n", sw.ToString ());
+			Assert.AreEqual ("\r\n  \r\n    <y><t yes-one-node=\"\">1</t><t not-node=\"\"></t></y>\r\n  \r\n".Replace ("\r\n", Environment.NewLine), sw.ToString ());
 		}
 
 		[Test]
@@ -2366,6 +2388,7 @@ NO
 		}
 
 		[Test]
+		[Category ("NotWorking")] // FIXME: SRE related regression
 		public void Bug487065 ()
 		{
 			using (XmlReader input = GetInput ()) {
@@ -2417,6 +2440,37 @@ NO
 		}
 
 		private bool valueHasBeenSet;
-#endif
+		
+		[Test] // bug #4434
+		public void IncludeProcessStylesheet ()
+		{
+			string includedXsl = @"<?xml version='1.0' ?>
+<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:doc='http://nwalsh.com/xsl/documentation/1.0' exclude-result-prefixes='doc' version='1.0'>
+	<doc:template name='foo' />
+	<xsl:template name='foo'>
+		<foo />
+	</xsl:template>
+</xsl:stylesheet>";
+			string file = Path.Combine (Path.GetTempPath (), "include.xsl");
+			StreamWriter includedWriter = new StreamWriter (file);
+			includedWriter.WriteLine (includedXsl);
+			includedWriter.Close ();
+			XslCompiledTransform transform = new XslCompiledTransform ();
+			string xsl = @"<?xml version='1.0' ?>
+<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform' version='1.0'>
+	<xsl:include href='include.xsl' />
+	<xsl:template match='/'>
+		<xsl:call-template name='foo' />
+	</xsl:template>
+</xsl:stylesheet>".Replace ("include.xsl", file);
+			XmlReader xslReader = XmlReader.Create (new StringReader (xsl));
+			transform.Load (xslReader);
+			XmlReader inputReader = XmlReader.Create (new StringReader ("<bar />"));
+			var sw = new StringWriter ();
+			XmlWriter outputWriter = XmlWriter.Create (sw);
+			transform.Transform (inputReader, outputWriter);
+			outputWriter.Close ();
+			Assert.AreEqual ("<?xml version=\"1.0\" encoding=\"utf-16\"?><foo />", sw.ToString (), "#1");
+		}
 	}
 }
