@@ -419,9 +419,8 @@ namespace MonoTests.System.Xml
 			Assert.AreEqual ("urn:foo", r.BaseURI);
 		}
 
-#if NET_4_5
 		[Test]
-		[ExpectedException (typeof (InvalidOperationException))]
+		[ExpectedException (typeof (XmlException))]
 		public void ReadonlyAsync ()
 		{
 			var s = new XmlReaderSettings ();
@@ -444,6 +443,5 @@ namespace MonoTests.System.Xml
 			var r2 = XmlReader.Create (r, c);
 			Assert.IsTrue (r2.Settings.Async);
 		}
-#endif
 	}
 }
