@@ -10,9 +10,7 @@ using System;
 using System.IO;
 using System.Xml;
 using NUnit.Framework;
-#if NET_4_5
 using System.Reflection;
-#endif
 
 namespace MonoTests.System.Xml
 {
@@ -36,7 +34,6 @@ namespace MonoTests.System.Xml
 		}
 
 		[Test]
-		[Category ("NotDotNet")]
 		public void FileUri2 ()
 		{
 			Assert.AreEqual (resolver.ResolveUri (new Uri ("file://usr/local/src"), null).ToString (), "file://usr/local/src");
@@ -59,6 +56,8 @@ namespace MonoTests.System.Xml
 		}
 
 		[Test]
+		[Category ("NotDotNet")] // It should throw ArgumentNullException.
+		[Ignore(".NET implementation does not throw ArgumentNullException.")]
 		[ExpectedException (typeof (ArgumentNullException))]
 		public void ResolveUriWithNullArgs ()
 		{
@@ -74,7 +73,7 @@ namespace MonoTests.System.Xml
 		}
 
 		[Test]
-		[ExpectedException (typeof (ArgumentException))]
+		[ExpectedException (typeof (InvalidOperationException))]
 		public void GetEntityWithRelativeFileUri ()
 		{
 			resolver.GetEntity (new Uri ("file.txt", UriKind.Relative), null, typeof (Stream));
@@ -97,7 +96,6 @@ namespace MonoTests.System.Xml
 			Assert.AreEqual ("view:Standard.xslt", uri.AbsoluteUri, "#2");
 		}
 
-#if NET_4_5
 		[Test]
 		public void TestAsync ()
 		{
@@ -125,6 +123,5 @@ namespace MonoTests.System.Xml
 				Assert.IsTrue (ex is XmlException);
 			}
 		}
-#endif
 	}
 }
