@@ -6,9 +6,18 @@
 //
 // (c) Copyright 2006 Novell, Inc. (http://www.novell.com)
 //
-#if NET_2_0
+
+#if SECURITY_DEP
+#if MONO_SECURITY_ALIAS
+extern alias MonoSecurity;
+using MSI = MonoSecurity::Mono.Security.Interface;
+#else
+using MSI = Mono.Security.Interface;
+#endif
+#endif
 
 using System;
+using System.Globalization;
 using System.IO;
 using System.Net.Sockets;
 using System.Text;
@@ -16,7 +25,9 @@ using System.Threading;
 using System.Net.Cache;
 using System.Security.Cryptography.X509Certificates;
 using System.Net;
-
+using System.Net.Security;
+using System.Security.Authentication;
+using Mono.Net.Security;
 
 namespace System.Net
 {
@@ -25,12 +36,14 @@ namespace System.Net
 		Uri requestUri;
 		string file_name; // By now, used for upload
 		ServicePoint servicePoint;
-		Socket dataSocket;
-		NetworkStream controlStream;
+		Stream origDataStream;
+		Stream dataStream;
+		Stream controlStream;
 		StreamReader controlReader;
 		NetworkCredential credentials;
 		IPHostEntry hostEntry;
 		IPEndPoint localEndPoint;
+		IPEndPoint remoteEndPoint;
 		IWebProxy proxy;
 		int timeout = 100000;
 		int rwTimeout = 300000;
@@ -54,7 +67,9 @@ namespace System.Net
 		const string PasswordCommand = "PASS";
 		const string TypeCommand = "TYPE";
 		const string PassiveCommand = "PASV";
+		const string ExtendedPassiveCommand = "EPSV";
 		const string PortCommand = "PORT";
+		const string ExtendedPortCommand = "EPRT";
 		const string AbortCommand = "ABOR";
 		const string AuthCommand = "AUTH";
 		const string RestCommand = "REST";
@@ -93,6 +108,8 @@ namespace System.Net
 			WebRequestMethods.Ftp.UploadFileWithUniqueName // STUR
 			};
 
+		Encoding dataEncoding = Encoding.UTF8;
+
 		internal FtpWebRequest (Uri uri) 
 		{
 			this.requestUri = uri;
@@ -172,6 +189,7 @@ namespace System.Net
 			}
 		}
 
+#if !MOBILE
 		[MonoTODO]
 		public static new RequestCachePolicy DefaultCachePolicy
 		{
@@ -182,6 +200,7 @@ namespace System.Net
 				throw GetMustImplement ();
 			}
 		}
+#endif
 
 		public bool EnableSsl {
 			get {
@@ -246,9 +265,6 @@ namespace System.Net
 			}
 			set {
 				CheckRequestStarted ();
-				if (value == null)
-					throw new ArgumentNullException ();
-
 				proxy = value;
 			}
 		}
@@ -389,6 +405,7 @@ namespace System.Net
 						State = RequestState.Scheduled;
 
 					Thread thread = new Thread (ProcessRequest);
+					thread.IsBackground = true;
 					thread.Start ();
 				}
 			}
@@ -440,6 +457,7 @@ namespace System.Net
 
 			asyncResult = new FtpAsyncResult (callback, state);
 			Thread thread = new Thread (ProcessRequest);
+			thread.IsBackground = true;
 			thread.Start ();
 
 			return asyncResult;
@@ -510,7 +528,8 @@ namespace System.Net
 					asyncResult.SetCompleted (false, ftpResponse);
 				}
 				catch (Exception e) {
-					State = RequestState.Error;
+					if (!GetServicePoint ().UsesProxy)
+						State = RequestState.Error;
 					SetCompleteWithError (e);
 				}
 			}
@@ -548,7 +567,12 @@ namespace System.Net
 				if (local_path [0] == '/')
 					local_path = local_path.Substring (1);
 
-				Uri initial = new Uri ("ftp://dummy-host" + initial_path);
+				UriBuilder initialBuilder = new UriBuilder () {
+					Scheme  = "ftp",
+					Host    = "dummy-host",
+					Path    = initial_path,
+				};
+				Uri initial = initialBuilder.Uri;
 				result = new Uri (initial, local_path).LocalPath;
 			}
 
@@ -577,6 +601,19 @@ namespace System.Net
 
 		void ProcessMethod ()
 		{
+			ServicePoint sp = GetServicePoint ();
+			if (sp.UsesProxy) {
+				if (method != WebRequestMethods.Ftp.DownloadFile)
+					throw new NotSupportedException ("FTP+proxy only supports RETR");
+
+				HttpWebRequest req = (HttpWebRequest) WebRequest.Create (proxy.GetProxy (requestUri));
+				req.Address = requestUri;
+				requestState = RequestState.Finished;
+				WebResponse response = req.GetResponse ();
+				ftpResponse.Stream = new FtpDataStream (this, response.GetResponseStream (), true);
+				ftpResponse.StatusCode = FtpStatusCode.CommandOK;
+				return;
+			}
 			State = RequestState.Connecting;
 
 			ResolveHost ();
@@ -615,13 +652,18 @@ namespace System.Net
 		}
 
 		private void CloseControlConnection () {
-			SendCommand (QuitCommand);
-			controlStream.Close ();
+			if (controlStream != null) {
+				SendCommand (QuitCommand);
+				controlStream.Close ();
+				controlStream = null;
+			}
 		}
 
-		private void CloseDataConnection () {
-			if(dataSocket != null)
-				dataSocket.Close ();
+		internal void CloseDataConnection () {
+			if(origDataStream != null) {
+				origDataStream.Close ();
+				origDataStream = null;
+			}
 		}
 
 		private void CloseConnection () {
@@ -712,7 +754,7 @@ namespace System.Net
 			OpenDataConnection ();
 
 			State = RequestState.TransferInProgress;
-			requestStream = new FtpDataStream (this, dataSocket, false);
+			requestStream = new FtpDataStream (this, dataStream, false);
 			asyncResult.Stream = requestStream;
 		}
 
@@ -723,7 +765,7 @@ namespace System.Net
 			OpenDataConnection ();
 
 			State = RequestState.TransferInProgress;
-			ftpResponse.Stream = new FtpDataStream (this, dataSocket, true);
+			ftpResponse.Stream = new FtpDataStream (this, dataStream, true);
 		}
 
 		void CheckRequestStarted ()
@@ -739,14 +781,14 @@ namespace System.Net
 			foreach (IPAddress address in hostEntry.AddressList) {
 				sock = new Socket (address.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
 
-				IPEndPoint remote = new IPEndPoint (address, requestUri.Port);
+				remoteEndPoint = new IPEndPoint (address, requestUri.Port);
 
-				if (!ServicePoint.CallEndPointDelegate (sock, remote)) {
+				if (!ServicePoint.CallEndPointDelegate (sock, remoteEndPoint)) {
 					sock.Close ();
 					sock = null;
 				} else {
 					try {
-						sock.Connect (remote);
+						sock.Connect (remoteEndPoint);
 						localEndPoint = (IPEndPoint) sock.LocalEndPoint;
 						break;
 					} catch (SocketException exc) {
@@ -769,7 +811,11 @@ namespace System.Net
 
 			Authenticate ();
 			FtpStatus status = SendCommand ("OPTS", "utf8", "on");
-			// ignore status for OPTS
+			if ((int)status.StatusCode < 200 || (int)status.StatusCode > 300)
+				dataEncoding = Encoding.Default;
+			else
+				dataEncoding = Encoding.UTF8;
+
 			status = SendCommand (WebRequestMethods.Ftp.PrintWorkingDirectory);
 			initial_path = GetInitialPath (status);
 		}
@@ -797,52 +843,19 @@ namespace System.Net
 		}
 
 		// Probably we could do better having here a regex
-		Socket SetupPassiveConnection (string statusDescription)
+		Socket SetupPassiveConnection (string statusDescription, bool ipv6)
 		{
 			// Current response string
 			string response = statusDescription;
 			if (response.Length < 4)
 				throw new WebException ("Cannot open passive data connection");
-			
-			// Look for first digit after code
-			int i;
-			for (i = 3; i < response.Length && !Char.IsDigit (response [i]); i++)
-				;
-			if (i >= response.Length)
-				throw new WebException ("Cannot open passive data connection");
 
-			// Get six elements
-			string [] digits = response.Substring (i).Split (new char [] {','}, 6);
-			if (digits.Length != 6)
-				throw new WebException ("Cannot open passive data connection");
+			int port = ipv6 ? GetPortV6 (response) : GetPortV4 (response);
 
-			// Clean non-digits at the end of last element
-			int j;
-			for (j = digits [5].Length - 1; j >= 0 && !Char.IsDigit (digits [5][j]); j--)
-				;
-			if (j < 0)
-				throw new WebException ("Cannot open passive data connection");
-			
-			digits [5] = digits [5].Substring (0, j + 1);
-
-			IPAddress ip;
-			try {
-				ip = IPAddress.Parse (String.Join (".", digits, 0, 4));
-			} catch (FormatException) {
-				throw new WebException ("Cannot open passive data connection");
-			}
-
-			// Get the port
-			int p1, p2, port;
-			if (!Int32.TryParse (digits [4], out p1) || !Int32.TryParse (digits [5], out p2))
-				throw new WebException ("Cannot open passive data connection");
-
-			port = (p1 << 8) + p2; // p1 * 256 + p2
-			//port = p1 * 256 + p2;
 			if (port < IPEndPoint.MinPort || port > IPEndPoint.MaxPort)
 				throw new WebException ("Cannot open passive data connection");
 
-			IPEndPoint ep = new IPEndPoint (ip, port);
+			IPEndPoint ep = new IPEndPoint (remoteEndPoint.Address, port);
 			Socket sock = new Socket (ep.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
 			try {
 				sock.Connect (ep);
@@ -854,6 +867,87 @@ namespace System.Net
 			return sock;
 		}
 
+		// GetPortV4, GetPortV6, FormatAddress and FormatAddressV6 are copied from referencesource
+		// TODO: replace FtpWebRequest completely.
+		private int GetPortV4(string responseString)
+		{
+			string [] parsedList = responseString.Split(new char [] {' ', '(', ',', ')'});
+
+			// We need at least the status code and the port
+			if (parsedList.Length <= 7) {
+				throw new FormatException(SR.GetString(SR.net_ftp_response_invalid_format, responseString));
+			}
+
+			int index = parsedList.Length-1;
+			// skip the last non-number token (e.g. terminating '.')
+#if MONO
+			// the MS code expects \r\n here in parsedList[index],
+			// but we're stripping the EOL off earlier so the array contains
+			// an empty string here which would make Char.IsNumber throw
+			// TODO: this can be removed once we switch FtpWebRequest to referencesource
+			if (parsedList[index] == "" || !Char.IsNumber(parsedList[index], 0))
+#else
+			if (!Char.IsNumber(parsedList[index], 0))
+#endif
+				index--;
+
+			int port = Convert.ToByte(parsedList[index--], NumberFormatInfo.InvariantInfo);
+			port = port |
+				(Convert.ToByte(parsedList[index--], NumberFormatInfo.InvariantInfo) << 8);
+
+			return port;
+		}
+
+		private int GetPortV6(string responseString)
+		{
+			int pos1 = responseString.LastIndexOf("(");
+			int pos2 = responseString.LastIndexOf(")");
+			if (pos1 == -1 || pos2 <= pos1) 
+				throw new FormatException(SR.GetString(SR.net_ftp_response_invalid_format, responseString));
+
+			// addressInfo will contain a string of format "|||<tcp-port>|"
+			string addressInfo = responseString.Substring(pos1+1, pos2-pos1-1);
+
+			// Although RFC2428 recommends using "|" as the delimiter,
+			// It allows ASCII characters in range 33-126 inclusive.
+			// We should consider allowing the full range.
+
+			string [] parsedList = addressInfo.Split(new char [] {'|'});
+			if (parsedList.Length < 4)
+				throw new FormatException(SR.GetString(SR.net_ftp_response_invalid_format, responseString));
+			
+			return Convert.ToInt32(parsedList[3], NumberFormatInfo.InvariantInfo);
+		}
+
+		private String FormatAddress(IPAddress address, int Port )
+		{
+			byte [] localAddressInBytes = address.GetAddressBytes();
+
+			// produces a string in FTP IPAddress/Port encoding (a1, a2, a3, a4, p1, p2), for sending as a parameter
+			// to the port command.
+			StringBuilder sb = new StringBuilder(32);
+			foreach (byte element in localAddressInBytes) {
+				sb.Append(element);
+				sb.Append(',');
+			}
+			sb.Append(Port / 256 );
+			sb.Append(',');
+			sb.Append(Port % 256 );
+			return sb.ToString();
+		}
+
+		private string FormatAddressV6(IPAddress address, int port) {
+			StringBuilder sb = new StringBuilder(43); // based on max size of IPv6 address + port + seperators
+			String addressString = address.ToString();
+			sb.Append("|2|");
+			sb.Append(addressString);
+			sb.Append('|');
+			sb.Append(port.ToString(NumberFormatInfo.InvariantInfo));
+			sb.Append('|');
+			return sb.ToString();
+		}
+		//
+
 		Exception CreateExceptionFromResponse (FtpStatus status)
 		{
 			FtpWebResponse ftpResponse = new FtpWebResponse (this, requestUri, method, status);
@@ -892,18 +986,19 @@ namespace System.Net
 		Socket InitDataConnection ()
 		{
 			FtpStatus status;
-			
+			bool ipv6 = remoteEndPoint.AddressFamily == AddressFamily.InterNetworkV6;
+
 			if (usePassive) {
-				status = SendCommand (PassiveCommand);
-				if (status.StatusCode != FtpStatusCode.EnteringPassive) {
+				status = SendCommand (ipv6 ? ExtendedPassiveCommand : PassiveCommand);
+				if (status.StatusCode != (ipv6 ? (FtpStatusCode)229 : FtpStatusCode.EnteringPassive)) { // FtpStatusCode doesn't contain code 229 for EPSV so we need to cast...
 					throw CreateExceptionFromResponse (status);
 				}
 				
-				return SetupPassiveConnection (status.StatusDescription);
+				return SetupPassiveConnection (status.StatusDescription, ipv6);
 			}
 
 			// Open a socket to listen the server's connection
-			Socket sock = new Socket (AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
+			Socket sock = new Socket (remoteEndPoint.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
 			try {
 				sock.Bind (new IPEndPoint (localEndPoint.Address, 0));
 				sock.Listen (1); // We only expect a connection from server
@@ -915,12 +1010,10 @@ namespace System.Net
 			}
 
 			IPEndPoint ep = (IPEndPoint) sock.LocalEndPoint;
-			string ipString = ep.Address.ToString ().Replace ('.', ',');
-			int h1 = ep.Port >> 8; // ep.Port / 256
-			int h2 = ep.Port % 256;
 
-			string portParam = ipString + "," + h1 + "," + h2;
-			status = SendCommand (PortCommand, portParam);
+			var portParam = ipv6 ? FormatAddressV6 (ep.Address, ep.Port) : FormatAddress (ep.Address, ep.Port);
+
+			status = SendCommand (ipv6 ? ExtendedPortCommand : PortCommand, portParam);
 			
 			if (status.StatusCode != FtpStatusCode.CommandOK) {
 				sock.Close ();
@@ -954,7 +1047,10 @@ namespace System.Net
 				throw CreateExceptionFromResponse (status);
 
 			if (usePassive) {
-				dataSocket = s;
+				origDataStream = new NetworkStream (s, true);
+				dataStream = origDataStream;
+				if (EnableSsl)
+					ChangeToSSLSocket (ref dataStream);
 			}
 			else {
 
@@ -972,12 +1068,10 @@ namespace System.Net
 				}
 
 				s.Close ();
-				dataSocket = incoming;
-			}
-
-			if (EnableSsl) {
-				InitiateSecureConnection (ref controlStream);
-				controlReader = new StreamReader (controlStream, Encoding.ASCII);
+				origDataStream = new NetworkStream (incoming, true);
+				dataStream = origDataStream;
+				if (EnableSsl)
+					ChangeToSSLSocket (ref dataStream);
 			}
 
 			ftpResponse.UpdateStatus (status);
@@ -1009,6 +1103,17 @@ namespace System.Net
 			if (EnableSsl) {
 				InitiateSecureConnection (ref controlStream);
 				controlReader = new StreamReader (controlStream, Encoding.ASCII);
+				status = SendCommand ("PBSZ", "0");
+				int st = (int) status.StatusCode;
+				if (st < 200 || st >= 300)
+					throw CreateExceptionFromResponse (status);
+				// TODO: what if "PROT P" is denied by the server? What does MS do?
+				status = SendCommand ("PROT", "P");
+				st = (int) status.StatusCode;
+				if (st < 200 || st >= 300)
+					throw CreateExceptionFromResponse (status);
+
+				status = new FtpStatus (FtpStatusCode.SendUserCommand, "");
 			}
 			
 			if (status.StatusCode != FtpStatusCode.SendUserCommand)
@@ -1044,7 +1149,7 @@ namespace System.Net
 				commandString += " " + String.Join (" ", parameters);
 
 			commandString += EOL;
-			cmd = Encoding.ASCII.GetBytes (commandString);
+			cmd = dataEncoding.GetBytes (commandString);
 			try {
 				controlStream.Write (cmd, 0, cmd.Length);
 			} catch (IOException) {
@@ -1087,6 +1192,7 @@ namespace System.Net
 					string line = null;
 					string find = code.ToString() + ' ';
 					while (true){
+						line = null;
 						try {
 							line = controlReader.ReadLine();
 						} catch (IOException) {
@@ -1104,19 +1210,22 @@ namespace System.Net
 			}
 		}
 
-		private void InitiateSecureConnection (ref NetworkStream stream) {
+		private void InitiateSecureConnection (ref Stream stream) {
 			FtpStatus status = SendCommand (AuthCommand, "TLS");
-
-			if (status.StatusCode != FtpStatusCode.ServerWantsSecureSession) {
+			if (status.StatusCode != FtpStatusCode.ServerWantsSecureSession)
 				throw CreateExceptionFromResponse (status);
-			}
 
 			ChangeToSSLSocket (ref stream);
 		}
 
-		internal static bool ChangeToSSLSocket (ref NetworkStream stream) {
-#if TARGET_JVM
-			stream.ChangeToSSLSocket ();
+		internal bool ChangeToSSLSocket (ref Stream stream) {
+#if SECURITY_DEP
+			var provider = MonoTlsProviderFactory.GetProviderInternal ();
+			var settings = MSI.MonoTlsSettings.CopyDefaultSettings ();
+			settings.UseServicePointManagerCallback = true;
+			var sslStream = provider.CreateSslStream (stream, true, settings);
+			sslStream.AuthenticateAsClient (requestUri.Host, null, SslProtocols.Default, false);
+			stream = sslStream.AuthenticatedStream;
 			return true;
 #else
 			throw new NotImplementedException ();
@@ -1143,5 +1252,4 @@ namespace System.Net
 	}
 }
 
-#endif
 
