@@ -43,12 +43,8 @@ using System.Text;
 
 namespace System.Net 
 {
-#if MOONLIGHT
-	internal class HttpWebResponse : WebResponse, ISerializable, IDisposable {
-#else
 	[Serializable]
 	public class HttpWebResponse : WebResponse, ISerializable, IDisposable {
-#endif
 		Uri uri;
 		WebHeaderCollection webHeaders;
 		CookieCollection cookieCollection;
@@ -110,7 +106,7 @@ namespace System.Net
 			version = (Version) info.GetValue ("version", typeof (Version));
 			statusCode = (HttpStatusCode) info.GetValue ("statusCode", typeof (HttpStatusCode));
 		}
-		
+
 		// Properties
 		
 		public string CharacterSet {
@@ -159,6 +155,7 @@ namespace System.Net
 			}
 		}
 		
+		virtual
 		public CookieCollection Cookies {
 			get {
 				CheckDisposed ();
@@ -203,6 +200,7 @@ namespace System.Net
 			}
 		}
 		
+		virtual
 		public string Method {
 			get {
 				CheckDisposed ();
@@ -231,12 +229,14 @@ namespace System.Net
 			}
 		}
 		
+		virtual
 		public HttpStatusCode StatusCode {
 			get {
 				return statusCode; 
 			}
 		}
 		
+		virtual
 		public string StatusDescription {
 			get {
 				CheckDisposed ();
@@ -244,6 +244,12 @@ namespace System.Net
 			}
 		}
 
+		public override bool SupportsHeaders {
+			get {
+				return true;
+			}
+		}
+
 		// Methods
 		
 		public string GetResponseHeader (string headerName)
@@ -269,7 +275,7 @@ namespace System.Net
 			CheckDisposed ();
 			if (stream == null)
 				return Stream.Null;  
-			if (0 == String.Compare (method, "HEAD", true)) // see par 4.3 & 9.4
+			if (string.Equals (method, "HEAD", StringComparison.OrdinalIgnoreCase))  // see par 4.3 & 9.4
 				return Stream.Null;  
 
 			return stream;
@@ -311,14 +317,12 @@ namespace System.Net
 		void IDisposable.Dispose ()
 		{
 			Dispose (true);
-			GC.SuppressFinalize (this);  
 		}
-
-		void Dispose (bool disposing) 
+		
+		protected override void Dispose (bool disposing)
 		{
 			this.disposed = true;
-			if (disposing)
-				Close ();
+			base.Dispose (true);
 		}
 		
 		private void CheckDisposed () 
@@ -332,105 +336,33 @@ namespace System.Net
 			if (webHeaders == null)
 				return;
 
-			string [] values = webHeaders.GetValues ("Set-Cookie");
-			if (values != null) {
-				foreach (string va in values)
-					SetCookie (va);
+			//
+			// Don't terminate response reading on bad cookie value
+			//
+			string value;
+			CookieCollection cookies = null;
+			try {
+				value = webHeaders.Get ("Set-Cookie");
+				if (value != null)
+					cookies = cookie_container.CookieCutter (uri, HttpKnownHeaderNames.SetCookie, value, false);
+			} catch {
 			}
 
-			values = webHeaders.GetValues ("Set-Cookie2");
-			if (values != null) {
-				foreach (string va in values)
-					SetCookie2 (va);
-			}
-		}
+			try {
+				value = webHeaders.Get ("Set-Cookie2");
+				if (value != null) {
+					var cookies2 = cookie_container.CookieCutter (uri, HttpKnownHeaderNames.SetCookie2, value, false);
 
-		void SetCookie (string header)
-		{
-			string name, val;
-			Cookie cookie = null;
-			CookieParser parser = new CookieParser (header);
-
-			while (parser.GetNextNameValue (out name, out val)) {
-				if ((name == null || name == "") && cookie == null)
-					continue;
-
-				if (cookie == null) {
-					cookie = new Cookie (name, val);
-					continue;
-				}
-
-				name = name.ToUpper ();
-				switch (name) {
-				case "COMMENT":
-					if (cookie.Comment == null)
-						cookie.Comment = val;
-					break;
-				case "COMMENTURL":
-					if (cookie.CommentUri == null)
-						cookie.CommentUri = new Uri (val);
-					break;
-				case "DISCARD":
-					cookie.Discard = true;
-					break;
-				case "DOMAIN":
-					if (cookie.Domain == "")
-						cookie.Domain = val;
-					break;
-				case "HTTPONLY":
-					cookie.HttpOnly = true;
-					break;
-				case "MAX-AGE": // RFC Style Set-Cookie2
-					if (cookie.Expires == DateTime.MinValue) {
-						try {
-						cookie.Expires = cookie.TimeStamp.AddSeconds (UInt32.Parse (val));
-						} catch {}
+					if (cookies != null && cookies.Count != 0) {
+						cookies.Add (cookies2);
+					} else {
+						cookies = cookies2;
 					}
-					break;
-				case "EXPIRES": // Netscape Style Set-Cookie
-					if (cookie.Expires != DateTime.MinValue)
-						break;
-
-					cookie.Expires = CookieParser.TryParseCookieExpires (val);
-					break;
-				case "PATH":
-					cookie.Path = val;
-					break;
-				case "PORT":
-					if (cookie.Port == null)
-						cookie.Port = val;
-					break;
-				case "SECURE":
-					cookie.Secure = true;
-					break;
-				case "VERSION":
-					try {
-						cookie.Version = (int) UInt32.Parse (val);
-					} catch {}
-					break;
 				}
+			} catch {
 			}
 
-			if (cookie == null)
-				return;
-
-			if (cookieCollection == null)
-				cookieCollection = new CookieCollection ();
-
-			if (cookie.Domain == "")
-				cookie.Domain = uri.Host;
-
-			cookieCollection.Add (cookie);
-			if (cookie_container != null)
-				cookie_container.Add (uri, cookie);
-		}
-
-		void SetCookie2 (string cookies_str)
-		{
-			string [] cookies = cookies_str.Split (',');
-	
-			foreach (string cookie_str in cookies)
-				SetCookie (cookie_str);
+			this.cookieCollection = cookies;
 		}
 	}	
 }
