@@ -7,9 +7,9 @@
 //
 // Copyright (c) 2006,2007,2008 Novell, Inc. (http://www.novell.com)
 //
-#if NET_2_0
 using NUnit.Framework;
 using System;
+using System.Collections.Generic;
 using System.IO;
 using System.Net;
 using System.Net.Sockets;
@@ -21,27 +21,61 @@ namespace MonoTests.System.Net
 	[TestFixture]
 	public class FtpWebRequestTest
 	{
-		FtpWebRequest defaultRequest;
-		
-		[TestFixtureSetUp]
-		public void Init ()
-		{
-			defaultRequest = (FtpWebRequest) WebRequest.Create ("ftp://www.contoso.com");
+		FtpWebRequest _defaultRequest;
+		FtpWebRequest defaultRequest {
+			get { return _defaultRequest ?? (_defaultRequest = (FtpWebRequest) WebRequest.Create ("ftp://www.contoso.com")); }
 		}
 		
+		private string _tempDirectory;
+		private string _tempFile;
+
+		[SetUp]
+		public void SetUp ()
+		{
+			_tempDirectory = Path.Combine (Path.GetTempPath (), "MonoTests.System.Net.FileWebRequestTest");
+			_tempFile = Path.Combine (_tempDirectory, "FtpWebRequestTest.tmp");
+			if (!Directory.Exists (_tempDirectory)) {
+				Directory.CreateDirectory (_tempDirectory);
+			} else {
+				// ensure no files are left over from previous runs
+				string [] files = Directory.GetFiles (_tempDirectory, "*");
+				foreach (string file in files)
+					File.Delete (file);
+			}
+		}
+
+		[TearDown]
+		public void TearDown ()
+		{
+			if (Directory.Exists (_tempDirectory))
+				Directory.Delete (_tempDirectory, true);
+		}
+
 		[Test]
 		public void ContentLength ()
 		{
 			try {
 				long l = defaultRequest.ContentLength;
+#if FEATURE_NO_BSD_SOCKETS
+				Assert.Fail ("#1a");
+			} catch (PlatformNotSupportedException) {
+				// OK.
+#else
 			} catch (NotSupportedException) {
 				Assert.Fail ("#1"); // Not overriden
+#endif
 			}
 
 			try {
 				defaultRequest.ContentLength = 2;
+#if FEATURE_NO_BSD_SOCKETS
+				Assert.Fail ("#2a");
+			} catch (PlatformNotSupportedException) {
+				// OK.
+#else
 			} catch (NotSupportedException) {
 				Assert.Fail ("#2"); // Not overriden
+#endif
 			}
 		}
 
@@ -62,6 +96,9 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
 		public void ContentOffset ()
 		{
 			try {
@@ -72,6 +109,9 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
 		public void Credentials ()
 		{
 			try {
@@ -83,6 +123,9 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
 		public void Method ()
 		{
 			try {
@@ -120,6 +163,9 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
 		public void ReadWriteTimeout ()
 		{
 			try {
@@ -130,6 +176,9 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
 		public void Timeout ()
 		{
 			try {
@@ -140,6 +189,9 @@ namespace MonoTests.System.Net
 		}
 		
 		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
 		public void DefaultValues ()
 		{
 			FtpWebRequest request = (FtpWebRequest) WebRequest.Create ("ftp://www.contoso.com");
@@ -157,6 +209,9 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
 		public void RenameTo ()
 		{
 			try {
@@ -173,23 +228,45 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
-		public void UploadFile1 ()
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void UploadFile1_v4 ()
 		{
-			ServerPut sp = new ServerPut ();
+			UploadFile1 (false);
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void UploadFile1_v6 ()
+		{
+			if (!Socket.OSSupportsIPv6)
+				Assert.Ignore ("IPv6 not supported.");
+
+			UploadFile1 (true);
+		}
+
+		void UploadFile1 (bool ipv6)
+		{
+			ServerPut sp = new ServerPut (ipv6);
 			sp.Start ();
-			string uri = String.Format ("ftp://{0}:{1}/uploads/file.txt", sp.IPAddress, sp.Port);
+			string uri = String.Format ("ftp://{0}:{1}/uploads/file.txt", EncloseIPv6 (sp.IPAddress), sp.Port);
 			try {
 				FtpWebRequest ftp = (FtpWebRequest) WebRequest.Create (uri);
 				ftp.KeepAlive = false;
 				ftp.Timeout = 5000;
 				ftp.Method = WebRequestMethods.Ftp.UploadFile;
-				ftp.ContentLength = 1;
+				ftp.ContentLength = 10;
 				ftp.UseBinary = true;
 				Stream stream = ftp.GetRequestStream ();
-				stream.WriteByte (0);
+				for (int i = 0; i < 10; i++)
+					stream.WriteByte ((byte)i);
 				stream.Close ();
 				FtpWebResponse response = (FtpWebResponse) ftp.GetResponse ();
 				Assert.IsTrue ((int) response.StatusCode >= 200 && (int) response.StatusCode < 300, "UP#01");
+				Assert.AreEqual (10, sp.result.Count, "UP#02");
 				response.Close ();
 			} catch (Exception) {
 				if (!String.IsNullOrEmpty (sp.Where))
@@ -201,11 +278,68 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
-		public void DownloadFile1 ()
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void UploadFile_WebClient_v4 ()
 		{
-			ServerDownload sp = new ServerDownload ();
+			UploadFile_WebClient (false);
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void UploadFile_WebClient_v6 ()
+		{
+			if (!Socket.OSSupportsIPv6)
+				Assert.Ignore ("IPv6 not supported.");
+
+			UploadFile_WebClient (true);
+		}
+
+		public void UploadFile_WebClient (bool ipv6)
+		{
+			ServerPut sp = new ServerPut (ipv6);
+			File.WriteAllText (_tempFile, "0123456789");
 			sp.Start ();
-			string uri = String.Format ("ftp://{0}:{1}/file.txt", sp.IPAddress, sp.Port);
+
+			using (WebClient m_WebClient = new WebClient())
+			{
+				string uri = String.Format ("ftp://{0}:{1}/uploads/file.txt", EncloseIPv6 (sp.IPAddress), sp.Port);
+				
+				m_WebClient.UploadFile(uri, _tempFile);
+			}
+			Assert.AreEqual (10, sp.result.Count, "WebClient/Ftp#01");
+	    
+			sp.Stop ();
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void DownloadFile1_v4 ()
+		{
+			DownloadFile (new ServerDownload (false));
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void DownloadFile1_v6 ()
+		{
+			if (!Socket.OSSupportsIPv6)
+				Assert.Ignore ("IPv6 not supported.");
+
+			DownloadFile (new ServerDownload (true));
+		}
+
+		void DownloadFile (ServerDownload sp)
+		{
+			sp.Start ();
+			string uri = String.Format ("ftp://{0}:{1}/file.txt", EncloseIPv6 (sp.IPAddress), sp.Port);
 			try {
 				FtpWebRequest ftp = (FtpWebRequest) WebRequest.Create (uri);
 				ftp.KeepAlive = false;
@@ -229,11 +363,53 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
-		public void DeleteFile1 ()
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void DownloadFile2_v4 ()
 		{
-			ServerDeleteFile sp = new ServerDeleteFile ();
+			// Some embedded FTP servers in Industrial Automation Hardware report
+			// the PWD using backslashes, but allow forward slashes for CWD.
+			DownloadFile (new ServerDownload (@"\Users\someuser", "/Users/someuser/", false));
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void DownloadFile2_v6 ()
+		{
+			// Some embedded FTP servers in Industrial Automation Hardware report
+			// the PWD using backslashes, but allow forward slashes for CWD.
+			DownloadFile (new ServerDownload (@"\Users\someuser", "/Users/someuser/", true));
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void DeleteFile1_v4 ()
+		{
+			DeleteFile1 (false);
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void DeleteFile1_v6 ()
+		{
+			if (!Socket.OSSupportsIPv6)
+				Assert.Ignore ("IPv6 not supported.");
+
+			DeleteFile1 (true);
+		}
+
+		void DeleteFile1 (bool ipv6)
+		{
+			ServerDeleteFile sp = new ServerDeleteFile (ipv6);
 			sp.Start ();
-			string uri = String.Format ("ftp://{0}:{1}/file.txt", sp.IPAddress, sp.Port);
+			string uri = String.Format ("ftp://{0}:{1}/file.txt", EncloseIPv6 (sp.IPAddress), sp.Port);
 			try {
 				FtpWebRequest ftp = (FtpWebRequest) WebRequest.Create (uri);
 				Console.WriteLine (ftp.RequestUri);
@@ -255,11 +431,31 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
-		public void ListDirectory1 ()
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void ListDirectory1_v4 ()
 		{
-			ServerListDirectory sp = new ServerListDirectory ();
+			ListDirectory1 (false);
+		}
+
+		[Test]
+#if FEATURE_NO_BSD_SOCKETS
+		[ExpectedException (typeof (PlatformNotSupportedException))]
+#endif
+		public void ListDirectory1_v6 ()
+		{
+			if (!Socket.OSSupportsIPv6)
+				Assert.Ignore ("IPv6 not supported.");
+
+			ListDirectory1 (true);
+		}
+
+		void ListDirectory1 (bool ipv6)
+		{
+			ServerListDirectory sp = new ServerListDirectory (ipv6);
 			sp.Start ();
-			string uri = String.Format ("ftp://{0}:{1}/somedir/", sp.IPAddress, sp.Port);
+			string uri = String.Format ("ftp://{0}:{1}/somedir/", EncloseIPv6 (sp.IPAddress), sp.Port);
 			try {
 				FtpWebRequest ftp = (FtpWebRequest) WebRequest.Create (uri);
 				Console.WriteLine (ftp.RequestUri);
@@ -282,7 +478,20 @@ namespace MonoTests.System.Net
 			}
 		}
 
+		string EncloseIPv6 (IPAddress address)
+		{
+			if (address.AddressFamily == AddressFamily.InterNetwork)
+				return address.ToString ();
+			
+			return String.Format ("[{0}]", address.ToString ());
+		}
+
 		class ServerListDirectory : FtpServer {
+			public ServerListDirectory (bool ipv6)
+				: base (ipv6)
+			{
+			}
+
 			protected override void Run ()
 			{
 				Socket client = control.Accept ();
@@ -300,24 +509,12 @@ namespace MonoTests.System.Net
 				}
 
 				string str = reader.ReadLine ();
-				if (str != "PASV") {
-					Where = "PASV";
+				string resp = FormatPassiveResponse (str);
+				if (resp == null) {
 					client.Close ();
 					return;
 				}
-
-				IPEndPoint end_data = (IPEndPoint) data.LocalEndPoint;
-				byte [] addr_bytes = end_data.Address.GetAddressBytes ();
-				byte [] port = new byte [2];
-				port[0] = (byte) ((end_data.Port >> 8) & 255);
-				port[1] = (byte) (end_data.Port & 255);
-				StringBuilder sb = new StringBuilder ("227 Passive (");
-				foreach (byte b in addr_bytes) {
-					sb.AppendFormat ("{0},", b);	
-				}
-				sb.AppendFormat ("{0},", port [0]);	
-				sb.AppendFormat ("{0})", port [1]);	
-				writer.WriteLine (sb.ToString ());
+				writer.WriteLine (resp);
 				writer.Flush ();
 
 				str = reader.ReadLine ();
@@ -344,6 +541,11 @@ namespace MonoTests.System.Net
 		}
 
 		class ServerDeleteFile : FtpServer {
+			public ServerDeleteFile (bool ipv6)
+				: base (ipv6)
+			{
+			}
+
 			protected override void Run ()
 			{
 				Socket client = control.Accept ();
@@ -377,6 +579,21 @@ namespace MonoTests.System.Net
 		}
 
 		class ServerDownload : FtpServer {
+
+			string Pwd, Cwd;
+
+			public ServerDownload (bool ipv6)
+				: this (null, null, ipv6)
+			{
+			}
+
+			public ServerDownload (string pwd, string cwd, bool ipv6)
+				: base (ipv6)
+			{
+				Pwd = pwd ?? "/home/someuser";
+				Cwd = cwd ?? "/home/someuser/";
+			}
+
 			protected override void Run ()
 			{
 				Socket client = control.Accept ();
@@ -388,30 +605,18 @@ namespace MonoTests.System.Net
 					return;
 				}
 
-				if (!DoInitialDialog (writer, reader, "/home/someuser", "/home/someuser/")) {
+				if (!DoInitialDialog (writer, reader, Pwd, Cwd)) {
 					client.Close ();
 					return;
 				}
 
 				string str = reader.ReadLine ();
-				if (str != "PASV") {
-					Where = "PASV";
+				string resp = FormatPassiveResponse (str);
+				if (resp == null) {
 					client.Close ();
 					return;
 				}
-
-				IPEndPoint end_data = (IPEndPoint) data.LocalEndPoint;
-				byte [] addr_bytes = end_data.Address.GetAddressBytes ();
-				byte [] port = new byte [2];
-				port[0] = (byte) ((end_data.Port >> 8) & 255);
-				port[1] = (byte) (end_data.Port & 255);
-				StringBuilder sb = new StringBuilder ("227 Passive (");
-				foreach (byte b in addr_bytes) {
-					sb.AppendFormat ("{0},", b);	
-				}
-				sb.AppendFormat ("{0},", port [0]);	
-				sb.AppendFormat ("{0})", port [1]);	
-				writer.WriteLine (sb.ToString ());
+				writer.WriteLine (resp);
 				writer.Flush ();
 
 				str = reader.ReadLine ();
@@ -438,6 +643,13 @@ namespace MonoTests.System.Net
 		}
 
 		class ServerPut : FtpServer {
+			public List<byte> result = new List<byte> ();
+			
+			public ServerPut (bool ipv6)
+				: base (ipv6)
+			{
+			}
+
 			protected override void Run ()
 			{
 				Socket client = control.Accept ();
@@ -455,24 +667,12 @@ namespace MonoTests.System.Net
 				}
 
 				string str = reader.ReadLine ();
-				if (str != "PASV") {
-					Where = "PASV";
+				string resp = FormatPassiveResponse (str);
+				if (resp == null) {
 					client.Close ();
 					return;
 				}
-
-				IPEndPoint end_data = (IPEndPoint) data.LocalEndPoint;
-				byte [] addr_bytes = end_data.Address.GetAddressBytes ();
-				byte [] port = new byte [2];
-				port[0] = (byte) ((end_data.Port >> 8) & 255);
-				port[1] = (byte) (end_data.Port & 255);
-				StringBuilder sb = new StringBuilder ("227 Passive (");
-				foreach (byte b in addr_bytes) {
-					sb.AppendFormat ("{0},", b);	
-				}
-				sb.AppendFormat ("{0},", port [0]);	
-				sb.AppendFormat ("{0})", port [1]);	
-				writer.WriteLine (sb.ToString ());
+				writer.WriteLine (resp);
 				writer.Flush ();
 
 				str = reader.ReadLine ();
@@ -485,8 +685,12 @@ namespace MonoTests.System.Net
 				writer.Flush ();
 
 				Socket data_cnc = data.Accept ();
-				byte [] dontcare = new byte [1];
-				data_cnc.Receive (dontcare, 1, SocketFlags.None);
+				var datastr = new NetworkStream (data_cnc, false);
+				int ch;
+				while ((ch = datastr.ReadByte ()) != -1){
+					result.Add ((byte)ch);
+
+				}
 				data_cnc.Close ();
 				writer.WriteLine ("226 File received Ok");
 				writer.Flush ();
@@ -502,16 +706,18 @@ namespace MonoTests.System.Net
 			protected Socket control;
 			protected Socket data;
 			protected ManualResetEvent evt;
+			protected bool ipv6;
 			public string Where = "";
 
-			public FtpServer ()
+			public FtpServer (bool ipv6)
 			{
-				control = new Socket (AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
-				control.Bind (new IPEndPoint (IPAddress.Loopback, 0));
+				control = new Socket (ipv6 ? AddressFamily.InterNetworkV6 : AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
+				control.Bind (new IPEndPoint (ipv6 ? IPAddress.IPv6Loopback : IPAddress.Loopback, 0));
 				control.Listen (1);
-				data = new Socket (AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
-				data.Bind (new IPEndPoint (IPAddress.Loopback, 0));
+				data = new Socket (ipv6 ? AddressFamily.InterNetworkV6 : AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
+				data.Bind (new IPEndPoint (ipv6 ? IPAddress.IPv6Loopback : IPAddress.Loopback, 0));
 				data.Listen (1);
+				this.ipv6 = ipv6;
 			}
 
 			public void Start ()
@@ -595,7 +801,39 @@ namespace MonoTests.System.Net
 				writer.Flush ();
 				return true;
 			}
-			
+
+			protected string FormatPassiveResponse (string request)
+			{
+				if (ipv6) {
+					if (request != "EPSV") {
+						Where = "EPSV";
+						return null;
+					}
+
+					IPEndPoint end_data = (IPEndPoint) data.LocalEndPoint;
+					return String.Format ("229 Extended Passive (|||{0}|)", end_data.Port);
+				}
+				else {
+					if (request != "PASV") {
+						Where = "PASV";
+						return null;
+					}
+
+					IPEndPoint end_data = (IPEndPoint) data.LocalEndPoint;
+					byte [] addr_bytes = end_data.Address.GetAddressBytes ();
+					byte [] port = new byte [2];
+					port[0] = (byte) ((end_data.Port >> 8) & 255);
+					port[1] = (byte) (end_data.Port & 255);
+					StringBuilder sb = new StringBuilder ("227 Passive (");
+					foreach (byte b in addr_bytes) {
+						sb.AppendFormat ("{0},", b);	
+					}
+					sb.AppendFormat ("{0},", port [0]);	
+					sb.AppendFormat ("{0})", port [1]);	
+					return sb.ToString ();
+				}
+			}
+
 			public IPAddress IPAddress {
 				get { return ((IPEndPoint) control.LocalEndPoint).Address; }
 			}
@@ -609,5 +847,4 @@ namespace MonoTests.System.Net
 	}
 }
 
-#endif
 
