@@ -21,21 +21,12 @@ namespace MonoTests.System
 	[TestFixture]
 	public class UriTest
 	{
-		protected bool isWin32 = false;
-		public bool IriParsing;
+		bool isWin32;
 
 		[TestFixtureSetUp]
 		public void GetReady ()
 		{
 			isWin32 = (Path.DirectorySeparatorChar == '\\');
-
-			//Make sure Uri static constructor is called
-			Uri.EscapeDataString ("");
-
-			FieldInfo iriParsingField = typeof (Uri).GetField ("s_IriParsing",
-				BindingFlags.Static | BindingFlags.GetField | BindingFlags.NonPublic);
-			if (iriParsingField != null)
-				IriParsing = (bool)iriParsingField.GetValue (null);
 		}
 
 		[Test]
@@ -605,10 +596,7 @@ namespace MonoTests.System
 			Uri u1 = new Uri("http://localhost:8080/test.aspx?ReturnUrl=%2fSearchDoc%2fSearcher.aspx");
 			Uri u2 = new Uri("http://localhost:8080/test.aspx?ReturnUrl=%252fSearchDoc%252fSearcher.aspx");
 
-			if (IriParsing)
-				Assert.AreEqual ("http://localhost:8080/test.aspx?ReturnUrl=%2fSearchDoc%2fSearcher.aspx", u1.ToString (), "QE1");
-			else
-				Assert.AreEqual ("http://localhost:8080/test.aspx?ReturnUrl=/SearchDoc/Searcher.aspx", u1.ToString (), "QE1");
+			Assert.AreEqual ("http://localhost:8080/test.aspx?ReturnUrl=%2fSearchDoc%2fSearcher.aspx", u1.ToString (), "QE1");
 
 			Assert.AreEqual ("http://localhost:8080/test.aspx?ReturnUrl=%252fSearchDoc%252fSearcher.aspx", u2.ToString (), "QE2");
 		}
@@ -848,11 +836,7 @@ namespace MonoTests.System
 		{
 			Uri u = new Uri("http://localhost/index.asp#main#start", false);
 
-#if NET_4_5
 				Assert.AreEqual (u.Fragment, "#main#start", "#1");
-#else
-				Assert.AreEqual (u.Fragment, "#main%23start", "#1");
-#endif
 
 			u = new Uri("http://localhost/index.asp#main#start", true);
 			Assert.AreEqual (u.Fragment, "#main#start", "#2");
@@ -861,11 +845,7 @@ namespace MonoTests.System
 
 			Uri b = new Uri ("http://www.gnome.org");
 			Uri n = new Uri (b, "blah#main#start");
-#if NET_4_5
 				Assert.AreEqual (n.Fragment, "#main#start", "#3");
-#else
-				Assert.AreEqual (n.Fragment, "#main%23start", "#3");
-#endif
 
 			n = new Uri (b, "blah#main#start", true);
 			Assert.AreEqual (n.Fragment, "#main#start", "#4");
@@ -897,11 +877,10 @@ namespace MonoTests.System
 		}
 
 		[Test]
-		[Category("NotDotNet")]
 		public void CheckHostName1 ()
 		{
 			// reported to MSDN Product Feedback Center (FDBK28671)
-			Assert.AreEqual (UriHostNameType.Unknown, Uri.CheckHostName (":11:22:33:44:55:66:77:88"), "#36 known to fail with ms.net: this is not a valid IPv6 address.");
+			Assert.AreEqual (UriHostNameType.IPv6, Uri.CheckHostName (":11:22:33:44:55:66:77:88"), "#36 known to fail with ms.net: this is not a valid IPv6 address.");
 		}
 
 		[Test]
@@ -1084,13 +1063,8 @@ namespace MonoTests.System
 			Uri ftp = new Uri ("FTP://[::ffFF:169.32.14.5]/");
 			Assert.AreEqual ("ftp", ftp.Scheme, "#7");
 
-#if NET_4_5
 			Assert.AreEqual ("[::ffff:169.32.14.5]", ftp.Host, "#8");
 			Assert.AreEqual ("ftp://[::ffff:169.32.14.5]/", ftp.ToString (), "#9");
-#else
-			Assert.AreEqual ("[0000:0000:0000:0000:0000:FFFF:A920:0E05]", ftp.Host, "#8");
-			Assert.AreEqual ("ftp://[0000:0000:0000:0000:0000:FFFF:A920:0E05]/", ftp.ToString (), "#9");
-#endif
 		}
 
 		[Test]
@@ -1211,14 +1185,6 @@ namespace MonoTests.System
 			Assert.AreEqual ("http://www.xxx.com/bar/foo/foobar.htm?z=0&y=5" + (char) 0xa9, uri1.MakeRelative (uri8), "#14");
 		}
 
-		[Test]
-		[Category ("NotWorking")]
-		public void RelativeUri ()
-		{
-			var u = new Uri ("/foo/bar");
-			Assert.IsFalse (u.IsAbsoluteUri, "#1");
-		}
-
 		[Test]
 		public void RelativeFragmentUri ()
 		{
@@ -1505,15 +1471,9 @@ namespace MonoTests.System
 			for (int i = 0; i < 128; i++)
 				sb.Append ((char) i);
 
-#if NET_4_5
 			Assert.AreEqual (
 				"%00%01%02%03%04%05%06%07%08%09%0A%0B%0C%0D%0E%0F%10%11%12%13%14%15%16%17%18%19%1A%1B%1C%1D%1E%1F%20%21%22%23%24%25%26%27%28%29%2A%2B%2C-.%2F0123456789%3A%3B%3C%3D%3E%3F%40ABCDEFGHIJKLMNOPQRSTUVWXYZ%5B%5C%5D%5E_%60abcdefghijklmnopqrstuvwxyz%7B%7C%7D~%7F",
 				Uri.EscapeDataString (sb.ToString ()));
-#else
-			Assert.AreEqual (
-				"%00%01%02%03%04%05%06%07%08%09%0A%0B%0C%0D%0E%0F%10%11%12%13%14%15%16%17%18%19%1A%1B%1C%1D%1E%1F%20!%22%23%24%25%26'()*%2B%2C-.%2F0123456789%3A%3B%3C%3D%3E%3F%40ABCDEFGHIJKLMNOPQRSTUVWXYZ%5B%5C%5D%5E_%60abcdefghijklmnopqrstuvwxyz%7B%7C%7D~%7F",
-				Uri.EscapeDataString (sb.ToString ()));
-#endif
 
 			Assert.AreEqual ("%C3%A1", Uri.EscapeDataString ("á"));
 		}
@@ -1524,15 +1484,9 @@ namespace MonoTests.System
 			for (int i = 0; i < 128; i++)
 				sb.Append ((char) i);
 
-#if NET_4_5
 			Assert.AreEqual (
 				"%00%01%02%03%04%05%06%07%08%09%0A%0B%0C%0D%0E%0F%10%11%12%13%14%15%16%17%18%19%1A%1B%1C%1D%1E%1F%20!%22#$%25&'()*+,-./0123456789:;%3C=%3E?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[%5C]%5E_%60abcdefghijklmnopqrstuvwxyz%7B%7C%7D~%7F",
 				Uri.EscapeUriString (sb.ToString ()));
-#else
-			Assert.AreEqual (
-				"%00%01%02%03%04%05%06%07%08%09%0A%0B%0C%0D%0E%0F%10%11%12%13%14%15%16%17%18%19%1A%1B%1C%1D%1E%1F%20!%22#$%25&'()*+,-./0123456789:;%3C=%3E?@ABCDEFGHIJKLMNOPQRSTUVWXYZ%5B%5C%5D%5E_%60abcdefghijklmnopqrstuvwxyz%7B%7C%7D~%7F",
-				Uri.EscapeUriString (sb.ToString ()));
-#endif
 
 			Assert.AreEqual ("%C3%A1", Uri.EscapeDataString ("á"));
 		}
@@ -1736,6 +1690,16 @@ namespace MonoTests.System
 			Assert.AreEqual ("file", unixuri.Scheme, "UnixAbsoluteFilePath_WithSpecialChars #2");
 		}
 
+		[Test]
+		[Category ("NotDotNet")]
+		public void UnixAbsolutePath_ReplaceRelative ()
+		{
+			var u1 = new Uri ("/Users/demo/Projects/file.xml");
+			var u2 = new Uri (u1, "b.jpg");
+
+			Assert.AreEqual ("file:///Users/demo/Projects/b.jpg", u2.ToString ());
+		}
+
 		[Test]
 		public void RelativeUriWithColons ()
 		{
@@ -1934,26 +1898,26 @@ namespace MonoTests.System
 				Uri uri;
 
 				uri = new Uri ("/foo", DotNetRelativeOrAbsolute);
-				Assert.IsFalse (uri.IsAbsoluteUri);
+				Assert.IsFalse (uri.IsAbsoluteUri, "#2");
 				
-				Uri.TryCreate("/foo", DotNetRelativeOrAbsolute, out uri);
-				Assert.IsFalse (uri.IsAbsoluteUri);
+				Assert.IsTrue (Uri.TryCreate("/foo", DotNetRelativeOrAbsolute, out uri), "#3");
+				Assert.IsFalse (uri.IsAbsoluteUri, "#3a");
 
 				if (useDotNetRelativeOrAbsoluteField != null) {
 					uri = new Uri ("/foo", UriKind.RelativeOrAbsolute);
-					Assert.IsTrue (uri.IsAbsoluteUri);
+					Assert.IsTrue (uri.IsAbsoluteUri, "#4");
 
-					Uri.TryCreate("/foo", UriKind.RelativeOrAbsolute, out uri);
-					Assert.IsTrue (uri.IsAbsoluteUri);
+					Assert.IsTrue (Uri.TryCreate("/foo", UriKind.RelativeOrAbsolute, out uri), "#5");
+					Assert.IsTrue (uri.IsAbsoluteUri, "#5a");
 
 					useDotNetRelativeOrAbsoluteField.SetValue (null, true);
 				}
 
 				uri = new Uri ("/foo", UriKind.RelativeOrAbsolute);
-				Assert.IsFalse (uri.IsAbsoluteUri);
+				Assert.IsFalse (uri.IsAbsoluteUri, "#10");
 
-				Uri.TryCreate("/foo", DotNetRelativeOrAbsolute, out uri);
-				Assert.IsFalse (uri.IsAbsoluteUri);
+				Assert.IsTrue (Uri.TryCreate("/foo", UriKind.RelativeOrAbsolute, out uri), "#11");
+				Assert.IsFalse (uri.IsAbsoluteUri, "#11a");
 			} finally {
 				if (useDotNetRelativeOrAbsoluteField != null)
 					useDotNetRelativeOrAbsoluteField.SetValue (null, useDotNetRelativeOrAbsoluteOld);
@@ -2038,42 +2002,57 @@ namespace MonoTests.System
 			var baseUri = new Uri ("http://test.com", UriKind.Absolute);
 			var relUri = new Uri ("path/dot./", UriKind.Relative);
 			var uri = new Uri (baseUri, relUri);
-			if (IriParsing)
-				Assert.AreEqual ("http://test.com/path/dot./", uri.ToString ());
-			else
-				Assert.AreEqual ("http://test.com/path/dot/", uri.ToString ());
-		}
-	}
-
-	// Tests non default IriParsing
-	[TestFixture]
-	public class UriTestAux : UriTest
-	{
-		private FieldInfo iriParsingField;
-		private bool originalIriParsing;
-
-		[TestFixtureSetUp]
-		public void GetReady2 ()
-		{
-			isWin32 = (Path.DirectorySeparatorChar == '\\');
-
-			//Make sure Uri static constructor is called
-			Uri.EscapeDataString ("");
-
-			iriParsingField = typeof (Uri).GetField ("s_IriParsing",
-				BindingFlags.Static | BindingFlags.GetField | BindingFlags.NonPublic);
-
-			originalIriParsing = (bool) iriParsingField.GetValue (null);
-
-			IriParsing = !originalIriParsing;
-
-			iriParsingField.SetValue (null, IriParsing);
+			Assert.AreEqual ("http://test.com/path/dot./", uri.ToString ());
 		}
 
-		[TestFixtureTearDown]
-		public void TearDown ()
+		[Test]
+		public void GuardedIPv6Address ()
 		{
-			iriParsingField.SetValue (null, originalIriParsing);
+			var x = new Uri ("asfd://[::1]:123/");
+			Assert.AreEqual ("[::1]", x.Host, "#1");
+		}
+
+		[Test]
+		public void CombineWithUserSchema ()
+		{
+			var baseUri = new Uri ("zip:mem:///");
+			var relativeUrl = "zip:mem:///foo/bar.txt";
+
+			var result = new Uri (baseUri, relativeUrl);
+
+			Assert.AreEqual ("zip:mem:///foo/bar.txt", result.ToString ());
+		}
+
+		[Test]
+		public void Scheme_msapp ()
+		{
+			var uri = new Uri ("ms-app://s-1-15-2-1613647288");
+			Assert.AreEqual ("ms-app", uri.Scheme);
+		}
+
+		[Test]
+		public void CombineWithUnixAbsolutePath ()
+		{
+			var a = new Uri ("http://localhost/");
+			var b = new Uri ("/foo", UriKind.RelativeOrAbsolute);
+			var res = new Uri (a, b);
+
+			Assert.AreEqual ("http://localhost/foo", res.ToString ());
+		}
+
+		[Test]
+		public void ImplicitUnixFileWithUnicode ()
+		{
+			string value = "/Library/Frameworks/System.Runtim…ee";
+			Uri uri;
+			Assert.IsTrue (Uri.TryCreate (value, UriKind.Absolute, out uri));
+		}
+
+		[Test]
+		public void UncValidPath ()
+		{
+			var uri = new Uri ("https://_foo/bar.html");
+			Assert.AreEqual ("https", uri.Scheme);
 		}
 	}
 }
