@@ -41,6 +41,7 @@ namespace System.Diagnostics {
 	[Serializable]
 	[ComVisible (true)]
 	[MonoTODO ("Serialized objects are not compatible with MS.NET")]
+	[StructLayout (LayoutKind.Sequential)]
         public class StackFrame {
 
                 public const int OFFSET_UNKNOWN = -1;
@@ -48,6 +49,8 @@ namespace System.Diagnostics {
 		#region Keep in sync with object-internals.h
 		private int ilOffset = OFFSET_UNKNOWN;
 		private int nativeOffset = OFFSET_UNKNOWN;
+		private long methodAddress;
+		private uint methodIndex;
 		private MethodBase methodBase;
 		private string fileName;
 		private int lineNumber;
@@ -68,22 +71,25 @@ namespace System.Diagnostics {
 					out nativeOffset, out fileName, out lineNumber,
 					out columnNumber);			
                 }
-                
+
+		[MethodImplAttribute (MethodImplOptions.NoInlining)]                
 		public StackFrame (bool fNeedFileInfo)
 		{
 			get_frame_info (2, fNeedFileInfo, out methodBase, out ilOffset,
 					out nativeOffset, out fileName, out lineNumber,
 					out columnNumber);			
                 }
-                
-                public StackFrame (int skipFrames)
+
+		[MethodImplAttribute (MethodImplOptions.NoInlining)]                
+		public StackFrame (int skipFrames)
 		{
 			get_frame_info (skipFrames + 2, false, out methodBase, out ilOffset,
 					out nativeOffset, out fileName, out lineNumber,
 					out columnNumber);			
                 }
                 
-                public StackFrame (int skipFrames, bool fNeedFileInfo) 
+		[MethodImplAttribute (MethodImplOptions.NoInlining)]
+		public StackFrame (int skipFrames, bool fNeedFileInfo) 
 		{
 			get_frame_info (skipFrames + 2, fNeedFileInfo, out methodBase, out ilOffset,
 					out nativeOffset, out fileName, out lineNumber,
@@ -92,7 +98,8 @@ namespace System.Diagnostics {
                 
 		// LAMESPEC: According to the MSDN docs, this creates a frame with _only_
 		// the filename and lineNumber, but MS fills out the frame info as well.
-                public StackFrame (string fileName, int lineNumber)
+		[MethodImplAttribute (MethodImplOptions.NoInlining)]
+		public StackFrame (string fileName, int lineNumber)
 		{
 			get_frame_info (2, false, out methodBase, out ilOffset,
 					out nativeOffset, out fileName, out lineNumber,
@@ -104,7 +111,8 @@ namespace System.Diagnostics {
                 
 		// LAMESPEC: According to the MSDN docs, this creates a frame with _only_
 		// the filename, lineNumber and colNumber, but MS fills out the frame info as well.
-                public StackFrame (string fileName, int lineNumber, int colNumber)
+		[MethodImplAttribute (MethodImplOptions.NoInlining)]
+		public StackFrame (string fileName, int lineNumber, int colNumber)
 		{
 			get_frame_info (2, false, out methodBase, out ilOffset,
 					out nativeOffset, out fileName, out lineNumber,
@@ -126,7 +134,7 @@ namespace System.Diagnostics {
                 
                 public virtual string GetFileName()
                 {
-#if !NET_2_1
+#if !MOBILE
 			if (SecurityManager.SecurityEnabled && (fileName != null) && (fileName.Length > 0)) {
 				string fn = Path.GetFullPath (fileName);
 				new FileIOPermission (FileIOPermissionAccess.PathDiscovery, fn).Demand ();
@@ -140,26 +148,12 @@ namespace System.Diagnostics {
 			string filename = "<filename unknown>";
 			if (fileName == null)
 				return filename;
-#if !MOONLIGHT
 			try {
 				filename = GetFileName ();
 			}
 			catch (SecurityException) {
 				// CAS check failure
 			}
-#else
-			// Silverlight always return <filename unknown> but that's not very useful for debugging
-			// OTOH we do not want to share any details about the original file system (even if they
-			// are likely available in the debugging symbols files) from the browser's plugin (but
-			// compiling stuff from smcs is fine since it's outside the sandbox)
-			try {
-				if (SecurityManager.SecurityEnabled)
-					filename = Path.GetFileName (fileName);
-			}
-			catch (ArgumentException) {
-				// e.g. invalid chars in filename
-			}
-#endif
 			return filename;
 		}
                 
@@ -178,6 +172,16 @@ namespace System.Diagnostics {
                         return nativeOffset;                        
                 }
 
+		internal long GetMethodAddress ()
+		{
+			return methodAddress;
+		}
+
+		internal uint GetMethodIndex ()
+		{
+			return methodIndex;
+		}
+
 		internal string GetInternalMethodName ()
 		{
 			return internalMethodName;
