@@ -31,6 +31,7 @@
 // (C) 2001 Ximian, Inc.  http://www.ximian.com
 //
 
+#if !FULL_AOT_RUNTIME
 using System;
 using System.Reflection;
 using System.Reflection.Emit;
@@ -39,12 +40,13 @@ using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
 
 namespace System.Reflection.Emit {
-#if NET_2_0
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_EnumBuilder))]
-#endif
 	[ClassInterface (ClassInterfaceType.None)]
-	public sealed class EnumBuilder : Type, _EnumBuilder {
+	public sealed class EnumBuilder : 
+		TypeInfo
+		, _EnumBuilder
+	{
 		private TypeBuilder _tb;
 		private FieldBuilder _underlyingField;
 		private Type _underlyingType;
@@ -64,6 +66,12 @@ namespace System.Reflection.Emit {
 			return _tb;
 		}
 
+		internal override Type InternalResolve ()
+		{
+			return _tb.InternalResolve (); 
+		}
+
+
 		public override Assembly Assembly {
 			get {
 				return _tb.Assembly;
@@ -154,16 +162,22 @@ namespace System.Reflection.Emit {
 			return res;
 		}
 
+		public TypeInfo CreateTypeInfo()
+		{
+			return _tb.CreateTypeInfo ();
+		}
+
+		public override Type GetEnumUnderlyingType ()
+		{
+			return _underlyingType;
+		}
+
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern void setup_enum_type (Type t);
 
 		public FieldBuilder DefineLiteral (string literalName, object literalValue)
 		{
-#if NET_2_0
 			Type fieldType = this;
-#else
-			Type fieldType = _underlyingType;
-#endif
 			FieldBuilder fieldBuilder = _tb.DefineField (literalName, 
 				fieldType, (FieldAttributes.Literal | 
 				(FieldAttributes.Static | FieldAttributes.Public)));
@@ -184,9 +198,7 @@ namespace System.Reflection.Emit {
 				modifiers);
 		}
 
-#if NET_2_0
 		[ComVisible (true)]
-#endif
 		public override ConstructorInfo[] GetConstructors(BindingFlags bindingAttr)
 		{
 			return _tb.GetConstructors (bindingAttr);
@@ -237,9 +249,7 @@ namespace System.Reflection.Emit {
 			return _tb.GetInterface (name, ignoreCase);
 		}
 
-#if NET_2_0
 		[ComVisible (true)]
-#endif
 		public override InterfaceMapping GetInterfaceMap (Type interfaceType)
 		{
 			return _tb.GetInterfaceMap (interfaceType);
@@ -351,40 +361,34 @@ namespace System.Reflection.Emit {
 			return _tb.IsDefined (attributeType, inherit);
 		}
 
-#if NET_2_0
-		[MonoTODO]
 		public override Type MakeArrayType ()
 		{
-			return base.MakeArrayType ();
+			return  new ArrayType (this, 0);
 		}
 
-		[MonoTODO]
 		public override Type MakeArrayType (int rank)
 		{
-			return base.MakeArrayType (rank);
+			if (rank < 1)
+				throw new IndexOutOfRangeException ();
+			return new ArrayType (this, rank);
 		}
 
-		[MonoTODO]
 		public override Type MakeByRefType ()
 		{
-			return base.MakeByRefType ();
+			return new ByRefType (this);
 		}
 
-		[MonoTODO]
 		public override Type MakePointerType ()
 		{
-			return base.MakePointerType ();
+			return new PointerType (this);
 		}
-#endif
 
 		public void SetCustomAttribute (CustomAttributeBuilder customBuilder)
 		{
 			_tb.SetCustomAttribute (customBuilder);
 		}
 
-#if NET_2_0
 		[ComVisible (true)]
-#endif
 		public void SetCustomAttribute (ConstructorInfo con, byte[] binaryAttribute)
 		{
 			SetCustomAttribute (new CustomAttributeBuilder (con, binaryAttribute));
@@ -414,5 +418,22 @@ namespace System.Reflection.Emit {
 		{
 			throw new NotImplementedException ();
 		}
+
+		internal override bool IsUserType {
+			get {
+				return false;
+			}
+		}
+
+		public override bool IsConstructedGenericType {
+			get { return false; }
+		}
+
+		public override bool IsAssignableFrom (TypeInfo typeInfo)
+		{
+			return base.IsAssignableFrom (typeInfo);
+		}
+
 	}
 }
+#endif
