@@ -41,7 +41,7 @@ namespace System
 {
 	public static partial class Console
 	{
-#if !NET_2_1
+#if !MOBILE
 		private class WindowsConsole
 		{
 			public static bool ctrlHandlerAdded = false;
@@ -100,7 +100,7 @@ namespace System
 				//
 				// On Windows, follow the Windows tradition
 				//
-#if NET_2_1
+#if MOBILE
 				// should never happen since Moonlight does not run on windows
 				inputEncoding = outputEncoding = Encoding.Default;
 #else			
@@ -134,47 +134,31 @@ namespace System
 
 		static void SetupStreams (Encoding inputEncoding, Encoding outputEncoding)
 		{
-#if !NET_2_1
+#if !MOBILE
 			if (!Environment.IsRunningOnWindows && ConsoleDriver.IsConsole) {
-				StreamWriter w = new CStreamWriter (OpenStandardOutput (0), outputEncoding, true);
-				w.AutoFlush = true;
-				stdout = TextWriter.Synchronized (w);
-
-				w = new CStreamWriter (OpenStandardOutput (0), outputEncoding, true);
-				w.AutoFlush = true;
-				stderr = TextWriter.Synchronized (w);
-				
 				stdin = new CStreamReader (OpenStandardInput (0), inputEncoding);
-			} else {
+				stdout = TextWriter.Synchronized (new CStreamWriter (OpenStandardOutput (0), outputEncoding, true) { AutoFlush = true });
+				stderr = TextWriter.Synchronized (new CStreamWriter (OpenStandardError (0), outputEncoding, true) { AutoFlush = true });
+			} else
 #endif
+			{
+				stdin = TextReader.Synchronized (new UnexceptionalStreamReader (OpenStandardInput (0), inputEncoding));
+
 #if MONOTOUCH
 				stdout = new NSLogWriter ();
-#else
-				stdout = new UnexceptionalStreamWriter (OpenStandardOutput (0), outputEncoding);
-				((StreamWriter)stdout).AutoFlush = true;
-#endif
-				stdout = TextWriter.Synchronized (stdout);
-
-#if MONOTOUCH
 				stderr = new NSLogWriter ();
 #else
-				stderr = new UnexceptionalStreamWriter (OpenStandardError (0), outputEncoding); 
-				((StreamWriter)stderr).AutoFlush = true;
-#endif
-				stderr = TextWriter.Synchronized (stderr);
-
-				stdin = new UnexceptionalStreamReader (OpenStandardInput (0), inputEncoding);
-				stdin = TextReader.Synchronized (stdin);
-#if !NET_2_1
-			}
-#endif
+				stdout = TextWriter.Synchronized (new UnexceptionalStreamWriter (OpenStandardOutput (0), outputEncoding) { AutoFlush = true });
+				stderr = TextWriter.Synchronized (new UnexceptionalStreamWriter (OpenStandardError (0), outputEncoding) { AutoFlush = true });
 
 #if MONODROID
-			if (LogcatTextWriter.IsRunningOnAndroid ()) {
-				stdout = TextWriter.Synchronized (new LogcatTextWriter ("mono-stdout", stdout));
-				stderr = TextWriter.Synchronized (new LogcatTextWriter ("mono-stderr", stderr));
+				if (LogcatTextWriter.IsRunningOnAndroid ()) {
+					stdout = TextWriter.Synchronized (new LogcatTextWriter ("mono-stdout", stdout));
+					stderr = TextWriter.Synchronized (new LogcatTextWriter ("mono-stderr", stderr));
+				}
+#endif // MONODROID
+#endif // MONOTOUCH
 			}
-#endif  // MONODROID
 
 			GC.SuppressFinalize (stdout);
 			GC.SuppressFinalize (stderr);
@@ -504,7 +488,7 @@ namespace System
 			stdout.WriteLine (String.Format (format, args));
 		}
 
-#if !NET_2_1
+#if !MOBILE
 		public static int Read ()
 		{
 			if ((stdin is CStreamReader) && ConsoleDriver.IsConsole) {
@@ -555,7 +539,7 @@ namespace System
 			}
 		}
 
-#if !NET_2_1
+#if !MOBILE
 		public static ConsoleColor BackgroundColor {
 			get { return ConsoleDriver.BackgroundColor; }
 			set { ConsoleDriver.BackgroundColor = value; }
@@ -798,6 +782,183 @@ namespace System
 			if (exit)
 				Environment.Exit (58);
 		}
+#else
+		// largely inspired by https://github.com/dotnet/corefx/blob/be8d2ce3964968cec9322a64211e37682085db70/src/System.Console/src/System/ConsolePal.WinRT.cs, because it's a similar platform where a console might not be available
+
+		// provide simply color tracking that allows round-tripping
+		internal const ConsoleColor UnknownColor = (ConsoleColor)(-1);
+		private static ConsoleColor s_trackedForegroundColor = UnknownColor;
+		private static ConsoleColor s_trackedBackgroundColor = UnknownColor;
+
+		public static ConsoleColor ForegroundColor
+		{
+			get
+			{
+				return s_trackedForegroundColor;
+			}
+			set
+			{
+				lock (Console.Out) // synchronize with other writers
+				{
+					s_trackedForegroundColor = value;
+				}
+			}
+		}
+
+		public static ConsoleColor BackgroundColor
+		{
+			get
+			{
+				return s_trackedBackgroundColor;
+			}
+			set
+			{
+				lock (Console.Out) // synchronize with other writers
+				{
+					s_trackedBackgroundColor = value;
+				}
+			}
+		}
+
+		public static int BufferWidth
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int BufferHeight
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static bool CapsLock { get { throw new PlatformNotSupportedException (); } }
+
+		public static int CursorLeft
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int CursorTop
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int CursorSize
+		{
+			get { return 100; }
+			set { throw new PlatformNotSupportedException(); }
+		}
+
+		public static bool CursorVisible
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static bool KeyAvailable { get { throw new PlatformNotSupportedException (); } }
+
+		public static int LargestWindowWidth
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int LargestWindowHeight
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static bool NumberLock { get { throw new PlatformNotSupportedException (); } }
+
+		public static string Title
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static bool TreatControlCAsInput
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int WindowHeight
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int WindowLeft
+		{
+			get { return 0; }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int WindowTop
+		{
+			get { return 0; }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static int WindowWidth
+		{
+			get { throw new PlatformNotSupportedException (); }
+			set { throw new PlatformNotSupportedException (); }
+		}
+
+		public static bool IsErrorRedirected { get { throw new PlatformNotSupportedException (); } }
+
+		public static bool IsInputRedirected { get { throw new PlatformNotSupportedException (); } }
+
+		public static bool IsOutputRedirected { get { throw new PlatformNotSupportedException (); } }
+
+		public static void Beep () { throw new PlatformNotSupportedException (); }
+
+		public static void Beep (int frequency, int duration) { throw new PlatformNotSupportedException (); }
+
+		public static void Clear () { throw new PlatformNotSupportedException (); }
+
+		public static void MoveBufferArea (int sourceLeft, int sourceTop, int sourceWidth, int sourceHeight, int targetLeft, int targetTop) { throw new PlatformNotSupportedException(); }
+
+		public static void MoveBufferArea (int sourceLeft, int sourceTop, int sourceWidth, int sourceHeight, int targetLeft, int targetTop, char sourceChar, ConsoleColor sourceForeColor, ConsoleColor sourceBackColor) { throw new PlatformNotSupportedException(); }
+
+
+		public static ConsoleKeyInfo ReadKey ()
+		{
+			return ReadKey (false);
+		}
+
+		public static ConsoleKeyInfo ReadKey (bool intercept) { throw new PlatformNotSupportedException (); }
+
+		public static void ResetColor ()
+		{
+			lock (Console.Out) // synchronize with other writers
+			{
+				s_trackedForegroundColor = UnknownColor;
+				s_trackedBackgroundColor = UnknownColor;
+			}
+		}
+
+		public static void SetBufferSize (int width, int height) { throw new PlatformNotSupportedException (); }
+
+		public static void SetCursorPosition (int left, int top) { throw new PlatformNotSupportedException (); }
+
+		public static void SetWindowPosition (int left, int top) { throw new PlatformNotSupportedException (); }
+
+		public static void SetWindowSize (int width, int height) { throw new PlatformNotSupportedException (); }
+
+		public static event ConsoleCancelEventHandler CancelKeyPress {
+			add {
+				throw new PlatformNotSupportedException ();
+			}
+			remove {
+				throw new PlatformNotSupportedException ();
+			}
+		}
 #endif
 	}
 }
