@@ -94,25 +94,6 @@ namespace MonoTests.System.Threading
 		//TimeSpan MaxValue = TimeSpan.FromMilliseconds ((long) Int32.MaxValue);
 		TimeSpan TooLarge = TimeSpan.FromMilliseconds ((long) Int32.MaxValue + 1);
 
-		static bool is_win32;
-		static bool is_mono;
-
-		static ThreadTest ()
-		{
-			switch (Environment.OSVersion.Platform) {
-			case PlatformID.Win32NT:
-			case PlatformID.Win32S:
-			case PlatformID.Win32Windows:
-			case PlatformID.WinCE:
-				is_win32 = true;
-				break;
-			}
-
-			// check a class in mscorlib to determine if we're running on Mono
-			if (Type.GetType ("System.MonoType", false) != null)
-				is_mono = true;
-		}
-
 		//Some Classes to test as threads
 		private class C1Test
 		{
@@ -327,8 +308,6 @@ namespace MonoTests.System.Threading
 		[Category ("NotDotNet")] // it hangs.
 		public void TestStart()
 		{
-			if (is_win32 && is_mono)
-				Assert.Fail ("This test fails on Win32. The test should be fixed.");
 		{
 			C1Test test1 = new C1Test();
 			Thread TestThread = new Thread(new ThreadStart(test1.TestMethod));
@@ -370,9 +349,6 @@ namespace MonoTests.System.Threading
 		[Test]
 		public void TestApartmentState ()
 		{
-			if (is_win32 && is_mono)
-				Assert.Fail ("This test fails on mono on win32. Our runtime should be fixed.");
-
 			C2Test test1 = new C2Test();
 			Thread TestThread = new Thread(new ThreadStart(test1.TestMethod));
 			Assert.AreEqual (ApartmentState.Unknown, TestThread.ApartmentState, "#1");
@@ -390,9 +366,6 @@ namespace MonoTests.System.Threading
 		[Category ("NotWorking")] // setting the priority of a Thread before it is started isn't implemented in Mono yet
 		public void TestPriority1()
 		{
-			if (is_win32 && is_mono)
-				Assert.Fail ("This test fails on mono on Win32. Our runtime should be fixed.");
-
 			C2Test test1 = new C2Test();
 			Thread TestThread = new Thread(new ThreadStart(test1.TestMethod));
 			try {
@@ -487,9 +460,6 @@ namespace MonoTests.System.Threading
 		[Test]
 		public void TestIsBackground1 ()
 		{
-			if (is_win32 && is_mono)
-				Assert.Fail ("This test fails on mono on Win32. Our runtime should be fixed.");
-
 			C2Test test1 = new C2Test();
 			Thread TestThread = new Thread(new ThreadStart(test1.TestMethod));
 			try {
@@ -534,9 +504,6 @@ namespace MonoTests.System.Threading
 		[Test]
 		public void TestName()
 		{
-			if (is_win32 && is_mono)
-				Assert.Fail ("This test fails on mono on Win32. Our runtime should be fixed.");
-
 			C2Test test1 = new C2Test();
 			Thread TestThread = new Thread(new ThreadStart(test1.TestMethod));
 			try {
@@ -574,37 +541,6 @@ namespace MonoTests.System.Threading
 			t.Name = "b";
 		}
 
-		bool rename_finished;
-		bool rename_failed;
-
-		[Test]
-		public void RenameTpThread ()
-		{
-			object monitor = new object ();
-			ThreadPool.QueueUserWorkItem (new WaitCallback (Rename_callback), monitor);
-			lock (monitor) {
-				if (!rename_finished)
-					Monitor.Wait (monitor);
-			}
-			Assert.IsTrue (!rename_failed);
-		}
-
-		void Rename_callback (object o) {
-			Thread.CurrentThread.Name = "a";
-			try {
-				Thread.CurrentThread.Name = "b";
-				//Console.WriteLine ("Thread name is: {0}", Thread.CurrentThread.Name);
-			} catch (Exception e) {
-				//Console.Error.WriteLine (e);
-				rename_failed = true;
-			}
-			object monitor = o;
-			lock (monitor) {
-				rename_finished = true;
-				Monitor.Pulse (monitor);
-			}
-		}
-
 		[Test]
 		public void TestNestedThreads1()
 		{
@@ -727,9 +663,6 @@ namespace MonoTests.System.Threading
 		[Test]
 		public void TestThreadState ()
 		{
-			if (is_win32 && is_mono)
-				Assert.Fail ("This test fails on mono on Win32. Our runtime should be fixed.");
-
 			//TODO: Test The rest of the possible transitions
 			C2Test test1 = new C2Test();
 			Thread TestThread = new Thread(new ThreadStart(test1.TestMethod));
@@ -861,9 +794,6 @@ namespace MonoTests.System.Threading
 		[Category("NotDotNet")] // On MS, ThreadStateException is thrown on Abort: "Thread is suspended; attempting to abort"
 		public void TestSuspendAbort ()
 		{
-			if (is_win32 && is_mono)
-				Assert.Fail ("This test fails on Win32. The test should be fixed.");
-
 			Thread t = new Thread (new ThreadStart (DoCount));
 			t.IsBackground = true;
 			t.Start ();
@@ -929,7 +859,6 @@ namespace MonoTests.System.Threading
 		}
 		
 		[Test]
-		[Category ("NotDotNet")] // it crashes nunit.
 		public void Test_InterruptCurrentThread ()
 		{
 			ManualResetEvent mre = new ManualResetEvent (false);
@@ -968,6 +897,7 @@ namespace MonoTests.System.Threading
 
 #if MONO_FEATURE_MULTIPLE_APPDOMAINS
 		[Test]
+		[Category ("NotDotNet")]
 		public void CurrentThread_Domains ()
 		{
 			AppDomain ad = AppDomain.CreateDomain ("foo");
