@@ -12,7 +12,7 @@
 
 using System;
 using System.Reflection;
-#if !MONOTOUCH
+#if !MONOTOUCH && !MOBILE_STATIC
 using System.Reflection.Emit;
 #endif
 using System.Runtime.InteropServices;
@@ -846,7 +846,7 @@ namespace MonoTests.System
 			Assert.IsTrue (custom [2].GetType () == typeof (SerializableAttribute));
 		}
 
-#if !MONOTOUCH
+#if !MONOTOUCH && !MOBILE_STATIC
 		[Test]
 		public void GetCustomAttributeOnNewSreTypes ()
 		{
@@ -903,7 +903,13 @@ namespace MonoTests.System
 			Assert.AreEqual (1, res.Length, "#1");
 		}
 
-		abstract class Abs
+		abstract class Root
+		{
+			[MyAttribute]
+			public abstract void Foo ();
+		}
+
+		abstract class Abs : Root
 		{
 			public abstract string Name { get; set; }
 		}
@@ -915,6 +921,8 @@ namespace MonoTests.System
 				get { return ""; }
 				set {}
 			}
+
+			public override void Foo () { }
 		}
 		
 		class Sub: Base
@@ -1032,6 +1040,27 @@ namespace MonoTests.System
 			a.GetHashCode ();
 		}
 
+
+		[Test]
+		public void DerivedClassOverrideHasInhertedAttributeFromAbstractRoot ()
+		{
+			// regression test for #44010
+			// we have
+			// abstract class Root {
+			//   [MyAttribute]
+			//   public abstract void Foo ();
+			// }
+			// abstract class Abs : Root { }
+			// class Base : Abs {
+			//   public override void  Foo () { }
+			// }
+			// note that Abs does not itself override Foo.
+			var bt = typeof(Base);
+			var m = bt.GetMethod ("Foo");
+			var attribute = Attribute.GetCustomAttribute (m, typeof (MyAttribute), true);
+			Assert.IsNotNull (attribute);
+		}
+
 		class ArrayAttribute : Attribute
 		{
 #pragma warning disable 414
