@@ -5,6 +5,7 @@
 //	Sebastien Pouliot <sebastien@ximian.com>
 //
 // (C) 2004-2005 Novell (http://www.novell.com)
+// Copyright 2011 Xamarin Inc (http://www.xamarin.com).
 //
 
 using System;
@@ -118,6 +119,15 @@ namespace MonoTests.System
 			ExpandStartsEnds ("Hello %%PATH%%%", "Hello %", "%%");
 		}
 
+		[Test]
+		// Bug #5169
+		public void ExpandEnvironmentVariables_ExpandMultiple ()
+		{
+			string path = Environment.GetEnvironmentVariable ("PATH");
+			var expected = "%TEST123" + path + "TEST456%";
+			ExpandEquals ("%TEST123%PATH%TEST456%", expected);
+		}
+
 		[Test]
 		public void GetEnvironmentVariables ()
 		{
@@ -128,7 +138,7 @@ namespace MonoTests.System
 			Assert.IsFalse (d.IsSynchronized, "IsSynchronized");
 		}
 
-#if NET_2_0 && !TARGET_JVM
+#if !MOBILE
 		[Test] // bug #333740
 		public void GetEnvironmentVariables_NewlySet ()
 		{
@@ -146,12 +156,16 @@ namespace MonoTests.System
 		public void GetCommandLineArgs ()
 		{
 			string[] args = Environment.GetCommandLineArgs ();
+#if !__WATCHOS__
 			Assert.IsNotNull (args, "not null");
 			Assert.IsTrue (((args.Length > 0) && (args.Length < 256)), "reasonable");
 			Assert.IsNotNull (args [0], "application");
+#else
+			Assert.AreEqual (0, args.Length, "length");
+#endif
 		}
 
-#if NET_2_0
+#if !MOBILE
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
 		public void GetEnvironmentVariable_Target_Invalid ()
@@ -166,7 +180,6 @@ namespace MonoTests.System
 			Environment.GetEnvironmentVariables ((EnvironmentVariableTarget)Int32.MinValue);
 		}
 
-#if !TARGET_JVM // Environment.SetEnvironmentVariable not supported under TARGET_JVM
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
 		public void SetEnvironmentVariable_Target_Invalid ()
@@ -218,7 +231,6 @@ namespace MonoTests.System
 			Environment.SetEnvironmentVariable ("A3", "\0");
 			Assert.IsNull (Environment.GetEnvironmentVariables ()["A3"]);
 		}
-#endif // TARGET_JVM
 #endif
 	}
 }
