@@ -18,7 +18,7 @@ using System.Diagnostics;
 using System.Text;
 using Mono.CompilerServices.SymbolWriter;
 
-#if NET_2_1
+#if MOBILE
 using XmlElement = System.Object;
 #else
 using System.Xml;
@@ -508,16 +508,6 @@ namespace Mono.CSharp {
 			return obsolete;
 		}
 
-		/// <summary>
-		/// Checks for ObsoleteAttribute presence. It's used for testing of all non-types elements
-		/// </summary>
-		public virtual void CheckObsoleteness (Location loc)
-		{
-			ObsoleteAttribute oa = GetAttributeObsolete ();
-			if (oa != null)
-				AttributeTester.Report_ObsoleteMessage (oa, GetSignatureForError (), loc, Report);
-		}
-
 		//
 		// Checks whether the type P is as accessible as this member
 		//
@@ -534,6 +524,9 @@ namespace Mono.CSharp {
 			while (TypeManager.HasElementType (p))
 				p = TypeManager.GetElementType (p);
 
+			if (p.BuiltinType != BuiltinTypeSpec.Type.None)
+				return true;
+
 			if (p.IsGenericParameter)
 				return true;
 
@@ -553,6 +546,10 @@ namespace Mono.CSharp {
 
 				bool same_access_restrictions = false;
 				for (MemberCore mc = this; !same_access_restrictions && mc != null && mc.Parent != null; mc = mc.Parent) {
+					var tc = mc as TypeContainer;
+					if (tc != null && tc.PartialContainer != null)
+						mc = tc.PartialContainer;
+
 					var al = mc.ModFlags & Modifiers.AccessibilityMask;
 					switch (pAccess) {
 					case Modifiers.INTERNAL:
@@ -573,8 +570,11 @@ namespace Mono.CSharp {
 							// protected type then the type is accessible
 							//
 							while (mc.Parent != null && mc.Parent.PartialContainer != null) {
-								if (mc.Parent.PartialContainer.IsBaseTypeDefinition (p_parent))
+								if (mc.Parent.PartialContainer.IsBaseTypeDefinition (p_parent)) {
 									same_access_restrictions = true;
+									break;
+								}
+
 								mc = mc.Parent; 
 							}
 						}
@@ -586,8 +586,15 @@ namespace Mono.CSharp {
 							same_access_restrictions = p.MemberDefinition.IsInternalAsPublic (mc.Module.DeclaringAssembly);
 						else if (al == (Modifiers.PROTECTED | Modifiers.INTERNAL))
 							same_access_restrictions = mc.Parent.PartialContainer.IsBaseTypeDefinition (p_parent) && p.MemberDefinition.IsInternalAsPublic (mc.Module.DeclaringAssembly);
-						else
+						else if (al == Modifiers.PROTECTED)
 							goto case Modifiers.PROTECTED;
+						else if (al == Modifiers.PRIVATE) {
+							if (p.MemberDefinition.IsInternalAsPublic (mc.Module.DeclaringAssembly)) {
+								same_access_restrictions = true;
+							} else {
+								goto case Modifiers.PROTECTED;
+							}
+						}
 
 						break;
 
@@ -1061,6 +1068,16 @@ namespace Mono.CSharp {
 
 		#endregion
 
+		public virtual void CheckObsoleteness (IMemberContext mc, Location loc)
+		{
+			var oa = GetAttributeObsolete ();
+			if (oa == null)
+				return;
+
+			if (!mc.IsObsolete)
+				AttributeTester.Report_ObsoleteMessage (oa, GetSignatureForError (), loc, mc.Module.Compiler.Report);
+		}
+
 		public virtual ObsoleteAttribute GetAttributeObsolete ()
 		{
 			if ((state & (StateFlags.Obsolete | StateFlags.Obsolete_Undetected)) == 0)
