@@ -406,8 +406,13 @@ namespace Mono.CSharp {
 
 		public static MemberSpec FindMember (TypeSpec container, MemberFilter filter, BindingRestriction restrictions)
 		{
+			if (filter.Kind == MemberKind.Method && container.Kind == MemberKind.TypeParameter && filter.Parameters == null)
+				throw new NotSupportedException ("type parameters methods cannot be lookup up due to two stage setup");
+
+			IList<MemberSpec> applicable;
+			var top_container = container;
+
 			do {
-				IList<MemberSpec> applicable;
 				if (container.MemberCache.member_hash.TryGetValue (filter.Name, out applicable)) {
 					// Start from the end because interface members are in reverse order
 					for (int i = applicable.Count - 1; i >= 0; i--) {
@@ -438,6 +443,26 @@ namespace Mono.CSharp {
 				container = container.BaseType;
 			} while (container != null);
 
+			var tps = top_container as TypeParameterSpec;
+			if (tps != null && tps.InterfaceCache != null) {
+				if (tps.InterfaceCache.member_hash.TryGetValue (filter.Name, out applicable)) {
+					for (int i = applicable.Count - 1; i >= 0; i--) {
+						var entry = applicable [i];
+
+						if ((restrictions & BindingRestriction.NoAccessors) != 0 && entry.IsAccessor)
+							continue;
+
+						if ((restrictions & BindingRestriction.OverrideOnly) != 0 && (entry.Modifiers & Modifiers.OVERRIDE) == 0)
+							continue;
+
+						if (!filter.Equals (entry))
+							continue;
+
+						return entry;
+					}
+				}
+			}
+
 			return null;
 		}
 
@@ -450,9 +475,9 @@ namespace Mono.CSharp {
 		//
 		public static IList<MemberSpec> FindMembers (TypeSpec container, string name, bool declaredOnlyClass)
 		{
-			IList<MemberSpec> applicable;
-
 			do {
+				IList<MemberSpec> applicable;
+				
 				if (container.MemberCache.member_hash.TryGetValue (name, out applicable) || declaredOnlyClass)
 					return applicable;
 
@@ -462,10 +487,21 @@ namespace Mono.CSharp {
 			return null;
 		}
 
+		public static IList<MemberSpec> FindInterfaceMembers (TypeParameterSpec typeParameter, string name)
+		{
+			if (typeParameter.InterfaceCache != null) {
+				IList<MemberSpec> applicable;
+				typeParameter.InterfaceCache.member_hash.TryGetValue (name, out applicable);
+				return applicable;
+			}
+
+			return null;
+		}
+
 		//
 		// Finds the nested type in container
 		//
-		public static TypeSpec FindNestedType (TypeSpec container, string name, int arity)
+		public static TypeSpec FindNestedType (TypeSpec container, string name, int arity, bool declaredOnlyClass)
 		{
 			IList<MemberSpec> applicable;
 			TypeSpec best_match = null;
@@ -494,7 +530,7 @@ namespace Mono.CSharp {
 						if (arity < 0) {
 							if (best_match == null) {
 								best_match = ts;
-							} else if (System.Math.Abs (ts.Arity + arity) < System.Math.Abs (ts.Arity + arity)) {
+							} else if (System.Math.Abs (ts.Arity + arity) < System.Math.Abs (best_match.Arity + arity)) {
 								best_match = ts;
 							}
 						}
@@ -502,7 +538,7 @@ namespace Mono.CSharp {
 				}
 
 				container = container.BaseType;
-			} while (container != null);
+			} while (container != null && !declaredOnlyClass);
 
 			return best_match;
 		}
@@ -643,6 +679,24 @@ namespace Mono.CSharp {
 			return ambig_candidate;
 		}
 
+		public static List<TypeSpec> GetDeclaredNestedTypes (TypeSpec container)
+		{
+			List<TypeSpec> found = null;
+			foreach (var entry in container.MemberCache.member_hash) {
+				foreach (var member in entry.Value) {
+					if ((member.Kind & MemberKind.NestedMask) == 0)
+						continue;
+
+					if (found == null)
+						found = new List<TypeSpec> ();
+
+					found.Add ((TypeSpec)member);
+				}
+			}
+
+			return found;
+		}
+
 		//
 		// Returns inflated version of MemberSpec, it works similarly to
 		// SRE TypeBuilder.GetMethod
