@@ -581,20 +581,20 @@ namespace Mono.CSharp {
 
 		public bool ProcessWarningsList (string text, Action<int> action)
 		{
-			bool valid = true;
 			foreach (string wid in text.Split (numeric_value_separator, StringSplitOptions.RemoveEmptyEntries)) {
+				var warning = wid;
+				if (warning.Length == 6 && warning [0] == 'C' && warning [1] == 'S')
+					warning = warning.Substring (2);
+
 				int id;
-				if (!int.TryParse (wid, NumberStyles.AllowLeadingWhite, CultureInfo.InvariantCulture, out id)) {
-					report.Error (1904, "`{0}' is not a valid warning number", wid);
-					valid = false;
+				if (!int.TryParse (warning, NumberStyles.AllowLeadingWhite, CultureInfo.InvariantCulture, out id)) {
 					continue;
 				}
 
-				if (report.CheckWarningCode (id, Location.Null))
-					action (id);
+				action (id);
 			}
 
-			return valid;
+			return true;
 		}
 
 		void Error_RequiresArgument (string option)
@@ -1195,7 +1195,7 @@ namespace Mono.CSharp {
 				}
 				return ParseResult.Success;
 
-			case "runtimemetadataversion":
+			case "/runtimemetadataversion":
 				if (value.Length == 0) {
 					Error_RequiresArgument (option);
 					return ParseResult.Error;
@@ -1204,6 +1204,36 @@ namespace Mono.CSharp {
 				settings.RuntimeMetadataVersion = value;
 				return ParseResult.Success;
 
+			// csc options that we don't support
+			case "/analyzer":
+			case "/appconfig":
+			case "/baseaddress":
+			case "/deterministic":
+			case "/deterministic+":
+			case "/deterministic-":
+			case "/errorendlocation":
+			case "/errorlog":
+			case "/features":
+			case "/highentropyva":
+			case "/highentropyva+":
+			case "/highentropyva-":
+			case "/link":
+			case "/moduleassemblyname":
+			case "/nowin32manifest":
+			case "/pathmap":
+			case "/pdb":
+			case "/preferreduilang":
+			case "/publicsign":
+			case "/publicsign+":
+			case "/publicsign-":
+			case "/reportanalyzer":
+			case "/ruleset":
+			case "/sqmsessionguid":
+			case "/subsystemversion":
+			case "/utf8output":
+			case "/win32manifest":
+				return ParseResult.Success;
+
 			default:
 				return ParseResult.UnknownOption;
 			}
@@ -1561,7 +1591,7 @@ namespace Mono.CSharp {
 				"   -help                Lists all compiler options (short: -?)\n" +
 				"   -keycontainer:NAME   The key pair container used to sign the output assembly\n" +
 				"   -keyfile:FILE        The key file used to strongname the ouput assembly\n" +
-				"   -langversion:TEXT    Specifies language version: ISO-1, ISO-2, 3, 4, 5, Default or Future\n" +
+				"   -langversion:TEXT    Specifies language version: ISO-1, ISO-2, 3, 4, 5, Default or Experimental\n" +
 				"   -lib:PATH1[,PATHn]   Specifies the location of referenced assemblies\n" +
 				"   -main:CLASS          Specifies the class with the Main method (short: -m)\n" +
 				"   -noconfig            Disables implicitly referenced assemblies\n" +
