@@ -69,29 +69,27 @@ namespace Mono.Linker.Steps {
 		protected virtual void InitializeAssembly (AssemblyDefinition assembly)
 		{
 			MarkAssembly (assembly);
-			foreach (TypeDefinition type in assembly.MainModule.Types) {
-				if (!Annotations.IsMarked (type))
-					continue;
 
+			foreach (TypeDefinition type in assembly.MainModule.Types)
 				InitializeType (type);
-			}
 		}
 
 		void InitializeType (TypeDefinition type)
 		{
+			if (type.HasNestedTypes) {
+				foreach (var nested in type.NestedTypes)
+					InitializeType (nested);
+			}
+
+			if (!Annotations.IsMarked (type))
+				return;
+
 			MarkType (type);
 
 			if (type.HasFields)
 				InitializeFields (type);
 			if (type.HasMethods)
 				InitializeMethods (type.Methods);
-
-			if (type.HasNestedTypes) {
-				foreach (var nested in type.NestedTypes) {
-					if (Annotations.IsMarked (nested))
-						InitializeType (nested);
-				}
-			}
 		}
 
 		void InitializeFields (TypeDefinition type)
@@ -141,8 +139,11 @@ namespace Mono.Linker.Steps {
 
 		void ProcessVirtualMethods ()
 		{
-			foreach (MethodDefinition method in _virtual_methods)
+			foreach (MethodDefinition method in _virtual_methods) {
+				Annotations.Push (method);
 				ProcessVirtualMethod (method);
+				Annotations.Pop ();
+			}
 		}
 
 		void ProcessVirtualMethod (MethodDefinition method)
@@ -193,17 +194,21 @@ namespace Mono.Linker.Steps {
 
 		protected virtual void MarkCustomAttribute (CustomAttribute ca)
 		{
+			Annotations.Push (ca);
 			MarkMethod (ca.Constructor);
 
 			MarkCustomAttributeArguments (ca);
 
 			TypeReference constructor_type = ca.Constructor.DeclaringType;
 			TypeDefinition type = constructor_type.Resolve ();
-			if (type == null)
+			if (type == null) {
+				Annotations.Pop ();
 				throw new ResolutionException (constructor_type);
+			}
 
 			MarkCustomAttributeProperties (ca, type);
 			MarkCustomAttributeFields (ca, type);
+			Annotations.Pop ();
 		}
 
 		protected void MarkSecurityDeclarations (ISecurityDeclarationProvider provider)
@@ -268,10 +273,12 @@ namespace Mono.Linker.Steps {
 		protected void MarkCustomAttributeProperty (CustomAttributeNamedArgument namedArgument, TypeDefinition attribute)
 		{
 			PropertyDefinition property = GetProperty (attribute, namedArgument.Name);
+			Annotations.Push (property);
 			if (property != null)
 				MarkMethod (property.SetMethod);
 
 			MarkIfType (namedArgument.Argument);
+			Annotations.Pop ();
 		}
 
 		PropertyDefinition GetProperty (TypeDefinition type, string propertyname)
@@ -365,7 +372,7 @@ namespace Mono.Linker.Steps {
 			// e.g. System.String[] -> System.String
 			var ts = (type as TypeSpecification);
 			if (ts != null) {
-				MarkWithResolvedScope (ts.GetElementType ());
+				MarkWithResolvedScope (ts.ElementType);
 				return;
 			}
 
@@ -511,13 +518,14 @@ namespace Mono.Linker.Steps {
 				MarkFields (type, type.IsEnum);
 
 			if (type.HasInterfaces) {
-				foreach (TypeReference iface in type.Interfaces)
-					MarkType (iface);
+				foreach (var iface in type.Interfaces)
+					MarkType (iface.InterfaceType);
 			}
 
 			if (type.HasMethods) {
 				MarkMethodsIf (type.Methods, IsVirtualAndHasPreservedParent);
 				MarkMethodsIf (type.Methods, IsStaticConstructorPredicate);
+				MarkMethodsIf (type.Methods, HasSerializationAttribute);
 			}
 
 			DoAdditionalTypeProcessing (type);
@@ -634,8 +642,10 @@ namespace Mono.Linker.Steps {
 				if (property.Name != property_name)
 					continue;
 
+				Annotations.Push (property);
 				MarkMethod (property.GetMethod);
 				MarkMethod (property.SetMethod);
+				Annotations.Pop ();
 			}
 		}
 
@@ -695,8 +705,11 @@ namespace Mono.Linker.Steps {
 		void MarkMethodsIf (ICollection methods, MethodPredicate predicate)
 		{
 			foreach (MethodDefinition method in methods)
-				if (predicate (method))
+				if (predicate (method)) {
+					Annotations.Push (predicate);
 					MarkMethod (method);
+					Annotations.Pop ();
+				}
 		}
 
 		static MethodPredicate IsDefaultConstructorPredicate = new MethodPredicate (IsDefaultConstructor);
@@ -726,6 +739,25 @@ namespace Mono.Linker.Steps {
 			return method.IsConstructor && method.IsStatic;
 		}
 
+		static bool HasSerializationAttribute (MethodDefinition method)
+		{
+			if (!method.HasCustomAttributes)
+				return false;
+			foreach (var ca in method.CustomAttributes) {
+				var cat = ca.AttributeType;
+				if (cat.Namespace != "System.Runtime.Serialization")
+					continue;
+				switch (cat.Name) {
+				case "OnDeserializedAttribute":
+				case "OnDeserializingAttribute":
+				case "OnSerializedAttribute":
+				case "OnSerializingAttribute":
+					return true;
+				}
+			}
+			return false;
+		}
+
 		static bool IsSerializable (TypeDefinition td)
 		{
 			return (td.Attributes & TypeAttributes.Serializable) != 0;
@@ -906,6 +938,7 @@ namespace Mono.Linker.Steps {
 			EnqueueMethod (method);
 
 			Annotations.Pop ();
+			Annotations.AddDependency (method);
 
 			return method;
 		}
@@ -944,6 +977,7 @@ namespace Mono.Linker.Steps {
 			if (CheckProcessed (method))
 				return;
 
+			Annotations.Push (method);
 			MarkType (method.DeclaringType);
 			MarkCustomAttributes (method);
 			MarkSecurityDeclarations (method);
@@ -987,6 +1021,7 @@ namespace Mono.Linker.Steps {
 			Annotations.Mark (method);
 
 			ApplyPreserveMethods (method);
+			Annotations.Pop ();
 		}
 
 		// Allow subclassers to mark additional things when marking a method
