@@ -10,6 +10,7 @@
  * Copyright 2001-2003 Ximian, Inc (http://www.ximian.com)
  * Copyright 2004-2009 Novell, Inc (http://www.novell.com)
  * Copyright 2011 Xamarin Inc. (http://www.xamarin.com)
+ * Licensed under the MIT license. See LICENSE file in the project root for full license information.
  */
 
 #include <config.h>
@@ -171,7 +172,7 @@ mono_mempool_stats (MonoMemPool *pool)
 {
 	MonoMemPool *p;
 	int count = 0;
-	guint32 still_free = pool->end - pool->pos;
+	guint32 still_free;
 
 	p = pool;
 	while (p) {
@@ -179,6 +180,7 @@ mono_mempool_stats (MonoMemPool *pool)
 		count++;
 	}
 	if (pool) {
+		still_free = pool->end - pool->pos;
 		g_print ("Mempool %p stats:\n", pool);
 		g_print ("Total mem allocated: %d\n", pool->d.allocated);
 		g_print ("Num chunks: %d\n", count);
@@ -196,25 +198,27 @@ static mono_mutex_t mempool_tracing_lock;
 static void
 mono_backtrace (int size)
 {
-        void *array[BACKTRACE_DEPTH];
-        char **names;
-        int i, symbols;
-        static gboolean inited;
+	void *array[BACKTRACE_DEPTH];
+	char **names;
+	int i, symbols;
+	static gboolean inited;
 
-        if (!inited) {
-            mono_os_mutex_init_recursive (&mempool_tracing_lock);
-            inited = TRUE;
-        }
+	if (!inited) {
+		mono_os_mutex_init_recursive (&mempool_tracing_lock);
+		inited = TRUE;
+	}
 
-        mono_os_mutex_lock (&mempool_tracing_lock);
-        g_print ("Allocating %d bytes\n", size);
-        symbols = backtrace (array, BACKTRACE_DEPTH);
-        names = backtrace_symbols (array, symbols);
-        for (i = 1; i < symbols; ++i) {
-                g_print ("\t%s\n", names [i]);
-        }
-        free (names);
-        mono_os_mutex_unlock (&mempool_tracing_lock);
+	mono_os_mutex_lock (&mempool_tracing_lock);
+	g_print ("Allocating %d bytes\n", size);
+	MONO_ENTER_GC_SAFE;
+	symbols = backtrace (array, BACKTRACE_DEPTH);
+	names = backtrace_symbols (array, symbols);
+	MONO_EXIT_GC_SAFE;
+	for (i = 1; i < symbols; ++i) {
+		g_print ("\t%s\n", names [i]);
+	}
+	g_free (names);
+	mono_os_mutex_unlock (&mempool_tracing_lock);
 }
 
 #endif
