@@ -7,6 +7,7 @@
  * Copyright 2001-2003 Ximian, Inc (http://www.ximian.com)
  * Copyright 2004-2009 Novell, Inc (http://www.novell.com)
  * Copyright 2011 Xamarin Inc (http://www.xamarin.com)
+ * Licensed under the MIT license. See LICENSE file in the project root for full license information.
  */
 
 #include <config.h>
@@ -21,6 +22,8 @@
 #include <mono/metadata/gc-internals.h>
 #include <mono/metadata/mempool.h>
 #include <mono/metadata/debug-mono-ppdb.h>
+#include <mono/metadata/exception-internals.h>
+#include <mono/metadata/runtime.h>
 #include <string.h>
 
 #define ALIGN_TO(val,align) ((((guint64)val) + ((align) - 1)) & ~((align) - 1))
@@ -458,23 +461,25 @@ mono_debug_add_method (MonoMethod *method, MonoDebugMethodJitInfo *jit, MonoDoma
 		write_sleb128 (lne->il_offset, ptr, &ptr);
 		write_sleb128 (lne->native_offset, ptr, &ptr);
 	}
+	write_leb128 (jit->has_var_info, ptr, &ptr);
+	if (jit->has_var_info) {
+		*ptr++ = jit->this_var ? 1 : 0;
+		if (jit->this_var)
+			write_variable (jit->this_var, ptr, &ptr);
 
-	*ptr++ = jit->this_var ? 1 : 0;
-	if (jit->this_var)
-		write_variable (jit->this_var, ptr, &ptr);
+		write_leb128 (jit->num_params, ptr, &ptr);
+		for (i = 0; i < jit->num_params; i++)
+			write_variable (&jit->params [i], ptr, &ptr);
 
-	write_leb128 (jit->num_params, ptr, &ptr);
-	for (i = 0; i < jit->num_params; i++)
-		write_variable (&jit->params [i], ptr, &ptr);
+		write_leb128 (jit->num_locals, ptr, &ptr);
+		for (i = 0; i < jit->num_locals; i++)
+			write_variable (&jit->locals [i], ptr, &ptr);
 
-	write_leb128 (jit->num_locals, ptr, &ptr);
-	for (i = 0; i < jit->num_locals; i++)
-		write_variable (&jit->locals [i], ptr, &ptr);
-
-	*ptr++ = jit->gsharedvt_info_var ? 1 : 0;
-	if (jit->gsharedvt_info_var) {
-		write_variable (jit->gsharedvt_info_var, ptr, &ptr);
-		write_variable (jit->gsharedvt_locals_var, ptr, &ptr);
+		*ptr++ = jit->gsharedvt_info_var ? 1 : 0;
+		if (jit->gsharedvt_info_var) {
+			write_variable (jit->gsharedvt_info_var, ptr, &ptr);
+			write_variable (jit->gsharedvt_locals_var, ptr, &ptr);
+		}
 	}
 
 	size = ptr - oldptr;
@@ -622,27 +627,29 @@ mono_debug_read_method (MonoDebugMethodAddress *address)
 		lne->il_offset = read_sleb128 (ptr, &ptr);
 		lne->native_offset = read_sleb128 (ptr, &ptr);
 	}
+	jit->has_var_info = read_leb128 (ptr, &ptr);
+	if (jit->has_var_info) {
+		if (*ptr++) {
+			jit->this_var = g_new0 (MonoDebugVarInfo, 1);
+			read_variable (jit->this_var, ptr, &ptr);
+		}
 
-	if (*ptr++) {
-		jit->this_var = g_new0 (MonoDebugVarInfo, 1);
-		read_variable (jit->this_var, ptr, &ptr);
-	}
+		jit->num_params = read_leb128 (ptr, &ptr);
+		jit->params = g_new0 (MonoDebugVarInfo, jit->num_params);
+		for (i = 0; i < jit->num_params; i++)
+			read_variable (&jit->params [i], ptr, &ptr);
 
-	jit->num_params = read_leb128 (ptr, &ptr);
-	jit->params = g_new0 (MonoDebugVarInfo, jit->num_params);
-	for (i = 0; i < jit->num_params; i++)
-		read_variable (&jit->params [i], ptr, &ptr);
+		jit->num_locals = read_leb128 (ptr, &ptr);
+		jit->locals = g_new0 (MonoDebugVarInfo, jit->num_locals);
+		for (i = 0; i < jit->num_locals; i++)
+			read_variable (&jit->locals [i], ptr, &ptr);
 
-	jit->num_locals = read_leb128 (ptr, &ptr);
-	jit->locals = g_new0 (MonoDebugVarInfo, jit->num_locals);
-	for (i = 0; i < jit->num_locals; i++)
-		read_variable (&jit->locals [i], ptr, &ptr);
-
-	if (*ptr++) {
-		jit->gsharedvt_info_var = g_new0 (MonoDebugVarInfo, 1);
-		jit->gsharedvt_locals_var = g_new0 (MonoDebugVarInfo, 1);
-		read_variable (jit->gsharedvt_info_var, ptr, &ptr);
-		read_variable (jit->gsharedvt_locals_var, ptr, &ptr);
+		if (*ptr++) {
+			jit->gsharedvt_info_var = g_new0 (MonoDebugVarInfo, 1);
+			jit->gsharedvt_locals_var = g_new0 (MonoDebugVarInfo, 1);
+			read_variable (jit->gsharedvt_info_var, ptr, &ptr);
+			read_variable (jit->gsharedvt_locals_var, ptr, &ptr);
+		}
 	}
 
 	return jit;
@@ -856,6 +863,14 @@ mono_debug_free_source_location (MonoDebugSourceLocation *location)
 	}
 }
 
+static int (*get_seq_point) (MonoDomain *domain, MonoMethod *method, gint32 native_offset);
+
+void
+mono_install_get_seq_point (MonoGetSeqPointFunc func)
+{
+	get_seq_point = func;
+}
+
 /**
  * mono_debug_print_stack_frame:
  * @native_offset: Native offset within the @method's machine code.
@@ -886,10 +901,22 @@ mono_debug_print_stack_frame (MonoMethod *method, guint32 native_offset, MonoDom
 			offset = -1;
 		}
 
+		if (offset < 0 && get_seq_point)
+			offset = get_seq_point (domain, method, native_offset);
+
 		if (offset < 0)
 			res = g_strdup_printf ("at %s <0x%05x>", fname, native_offset);
-		else
-			res = g_strdup_printf ("at %s <IL 0x%05x, 0x%05x>", fname, offset, native_offset);
+		else {
+			char *mvid = mono_guid_to_string_minimal ((uint8_t*)method->klass->image->heap_guid.data);
+			char *aotid = mono_runtime_get_aotid ();
+			if (aotid)
+				res = g_strdup_printf ("at %s [0x%05x] in <%s#%s>:0" , fname, offset, mvid, aotid);
+			else
+				res = g_strdup_printf ("at %s [0x%05x] in <%s>:0" , fname, offset, mvid);
+
+			g_free (aotid);
+			g_free (mvid);
+		}
 		g_free (fname);
 		return res;
 	}
