@@ -8,6 +8,7 @@
  * Copyright 2001-2003 Ximian, Inc (http://www.ximian.com)
  * Copyright 2004-2009 Novell, Inc (http://www.novell.com)
  * Copyright 2011 Xamarin Inc (http://www.xamarin.com).
+ * Licensed under the MIT license. See LICENSE file in the project root for full license information.
  */
 
 #include "config.h"
@@ -23,6 +24,7 @@
 #include "mono/metadata/mono-config-dirs.h"
 #include "mono/io-layer/io-layer.h"
 #include "mono/utils/mono-dl.h"
+#include <mono/utils/mono-logger-internals.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -100,6 +102,11 @@ struct _ProfilerDesc {
 	MonoProfileGCHandleFunc  gc_handle;
 	MonoProfileGCRootFunc    gc_roots;
 
+	MonoProfileGCFinalizeFunc gc_finalize_begin;
+	MonoProfileGCFinalizeObjectFunc gc_finalize_object_begin;
+	MonoProfileGCFinalizeObjectFunc gc_finalize_object_end;
+	MonoProfileGCFinalizeFunc gc_finalize_end;
+
 	MonoProfileFunc          runtime_initialized_event;
 
 	MonoProfilerCodeChunkNew code_chunk_new;
@@ -266,7 +273,6 @@ mono_profiler_install_transition (MonoProfileMethodResult callback)
 void 
 mono_profiler_install_allocation (MonoProfileAllocFunc callback)
 {
-	mono_gc_enable_alloc_events ();
 	if (!prof_list)
 		return;
 	prof_list->allocation_cb = callback;
@@ -281,7 +287,7 @@ mono_profiler_install_monitor  (MonoProfileMonitorFunc callback)
 }
 
 static MonoProfileSamplingMode sampling_mode = MONO_PROFILER_STAT_MODE_PROCESS;
-static int64_t sampling_frequency = 1000; //1ms
+static int64_t sampling_frequency = 100; // Hz
 
 /**
  * mono_profiler_set_statistical_mode:
@@ -296,10 +302,10 @@ static int64_t sampling_frequency = 1000; //1ms
  * Said that, when using statistical sampling, always assume variable rate sampling as all sort of external factors can interfere.
  */
 void
-mono_profiler_set_statistical_mode (MonoProfileSamplingMode mode, int64_t sampling_frequency_is_us)
+mono_profiler_set_statistical_mode (MonoProfileSamplingMode mode, int64_t sampling_frequency_hz)
 {
 	sampling_mode = mode;
-	sampling_frequency = sampling_frequency_is_us;
+	sampling_frequency = sampling_frequency_hz;
 }
 
 void 
@@ -869,7 +875,6 @@ mono_profiler_gc_roots (int num, void **objects, int *root_types, uintptr_t *ext
 void
 mono_profiler_install_gc (MonoProfileGCFunc callback, MonoProfileGCResizeFunc heap_resize_callback)
 {
-	mono_gc_enable_events ();
 	if (!prof_list)
 		return;
 	prof_list->gc_event = callback;
@@ -923,6 +928,50 @@ mono_profiler_install_gc_roots (MonoProfileGCHandleFunc handle_callback, MonoPro
 	prof_list->gc_roots = roots_callback;
 }
 
+void
+mono_profiler_gc_finalize_begin (void)
+{
+	for (ProfilerDesc *prof = prof_list; prof; prof = prof->next)
+		if ((prof->events & MONO_PROFILE_GC_FINALIZATION) && prof->gc_finalize_begin)
+			prof->gc_finalize_begin (prof->profiler);
+}
+
+void
+mono_profiler_gc_finalize_object_begin (MonoObject *obj)
+{
+	for (ProfilerDesc *prof = prof_list; prof; prof = prof->next)
+		if ((prof->events & MONO_PROFILE_GC_FINALIZATION) && prof->gc_finalize_object_begin)
+			prof->gc_finalize_object_begin (prof->profiler, obj);
+}
+
+void
+mono_profiler_gc_finalize_object_end (MonoObject *obj)
+{
+	for (ProfilerDesc *prof = prof_list; prof; prof = prof->next)
+		if ((prof->events & MONO_PROFILE_GC_FINALIZATION) && prof->gc_finalize_object_end)
+			prof->gc_finalize_object_end (prof->profiler, obj);
+}
+
+void
+mono_profiler_gc_finalize_end (void)
+{
+	for (ProfilerDesc *prof = prof_list; prof; prof = prof->next)
+		if ((prof->events & MONO_PROFILE_GC_FINALIZATION) && prof->gc_finalize_end)
+			prof->gc_finalize_end (prof->profiler);
+}
+
+void
+mono_profiler_install_gc_finalize (MonoProfileGCFinalizeFunc begin, MonoProfileGCFinalizeObjectFunc begin_obj, MonoProfileGCFinalizeObjectFunc end_obj, MonoProfileGCFinalizeFunc end)
+{
+	if (!prof_list)
+		return;
+
+	prof_list->gc_finalize_begin = begin;
+	prof_list->gc_finalize_object_begin = begin_obj;
+	prof_list->gc_finalize_object_end = end_obj;
+	prof_list->gc_finalize_end = end;
+}
+
 void
 mono_profiler_install_runtime_initialized (MonoProfileFunc runtime_initialized_callback)
 {
@@ -1063,6 +1112,7 @@ mono_profiler_coverage_free (MonoMethod *method)
 void 
 mono_profiler_coverage_get (MonoProfiler *prof, MonoMethod *method, MonoProfileCoverageFunc func)
 {
+	MonoError error;
 	MonoProfileCoverageInfo* info = NULL;
 	int i, offset;
 	guint32 code_size;
@@ -1079,7 +1129,8 @@ mono_profiler_coverage_get (MonoProfiler *prof, MonoMethod *method, MonoProfileC
 	if (!info)
 		return;
 
-	header = mono_method_get_header (method);
+	header = mono_method_get_header_checked (method, &error);
+	mono_error_assert_ok (&error);
 	start = mono_method_header_get_code (header, &code_size, NULL);
 	debug_minfo = mono_debug_lookup_method (method);
 
@@ -1143,7 +1194,15 @@ load_embedded_profiler (const char *desc, const char *name)
 	MonoDl *pmodule = NULL;
 	gboolean result;
 
-	pmodule = mono_dl_open (NULL, MONO_DL_LAZY, &err);
+	/*
+	 * Some profilers (such as ours) may need to call back into the runtime
+	 * from their sampling callback (which is called in async-signal context).
+	 * They need to be able to know that all references back to the runtime
+	 * have been resolved; otherwise, calling runtime functions may result in
+	 * invoking the dynamic linker which is not async-signal-safe. Passing
+	 * MONO_DL_EAGER will ask the dynamic linker to resolve everything upfront.
+	 */
+	pmodule = mono_dl_open (NULL, MONO_DL_EAGER, &err);
 	if (!pmodule) {
 		g_warning ("Could not open main executable (%s)", err);
 		g_free (err);
@@ -1157,6 +1216,7 @@ load_embedded_profiler (const char *desc, const char *name)
 	return result;
 }
 
+// TODO: Much of the library loading code here is custom. It would be better to merge this with mono-dl
 static gboolean
 load_profiler_from_directory (const char *directory, const char *libname, const char *desc)
 {
@@ -1165,10 +1225,13 @@ load_profiler_from_directory (const char *directory, const char *libname, const
 	char *err;
 	void *iter;
 
+	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_DLLIMPORT, "Attempting to load profiler %s from %s (desc %s)", libname, directory, desc);
+
 	iter = NULL;
 	err = NULL;
 	while ((path = mono_dl_build_path (directory, libname, &iter))) {
-		pmodule = mono_dl_open (path, MONO_DL_LAZY, &err);
+		pmodule = mono_dl_open (path, MONO_DL_EAGER, &err);
+		mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_DLLIMPORT, "Attempting to load profiler: %s, %ssuccessful, err: %s", path, pmodule?"":"not ", err);
 		g_free (path);
 		g_free (err);
 		if (pmodule)
@@ -1179,10 +1242,11 @@ load_profiler_from_directory (const char *directory, const char *libname, const
 }
 
 static gboolean
-load_profiler_from_mono_instalation (const char *libname, const char *desc)
+load_profiler_from_mono_installation (const char *libname, const char *desc)
 {
 	char *err = NULL;
-	MonoDl *pmodule = mono_dl_open_runtime_lib (libname, MONO_DL_LAZY, &err);
+	MonoDl *pmodule = mono_dl_open_runtime_lib (libname, MONO_DL_EAGER, &err);
+	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_DLLIMPORT, "Attempting to load profiler from runtime libs: %s, %ssuccessful, err: %s", libname, pmodule?"":"not ", err);
 	g_free (err);
 	if (pmodule)
 		return load_profiler (pmodule, desc, INITIALIZER_NAME);
@@ -1247,12 +1311,11 @@ mono_profiler_load (const char *desc)
 		}
 		if (!load_embedded_profiler (desc, mname)) {
 			libname = g_strdup_printf ("mono-profiler-%s", mname);
-			if (mono_config_get_assemblies_dir ())
+			res = load_profiler_from_mono_installation (libname, desc);
+			if (!res && mono_config_get_assemblies_dir ())
 				res = load_profiler_from_directory (mono_assembly_getrootdir (), libname, desc);
 			if (!res)
 				res = load_profiler_from_directory (NULL, libname, desc);
-			if (!res)
-				res = load_profiler_from_mono_instalation (libname, desc);
 			if (!res)
 				g_warning ("The '%s' profiler wasn't found in the main executable nor could it be loaded from '%s'.", mname, libname);
 			g_free (libname);
