@@ -17,10 +17,12 @@
 
 MONO_BEGIN_DECLS
 
-gboolean mono_runtime_is_critical_method (MonoMethod *method) MONO_INTERNAL;
-gboolean mono_runtime_try_shutdown (void) MONO_INTERNAL;
+gboolean mono_runtime_is_critical_method (MonoMethod *method);
+gboolean mono_runtime_try_shutdown (void);
 
-void mono_runtime_init_tls (void) MONO_INTERNAL;
+void mono_runtime_init_tls (void);
+
+char* mono_runtime_get_aotid (void);
 MONO_END_DECLS
 
 #endif /* _MONO_METADATA_RUNTIME_H_ */
