@@ -4,33 +4,49 @@
 #include <glib.h>
 
 #include "mono-compiler.h"
-#include "mono-logger-internal.h"
+#include "mono-logger-internals.h"
 
 typedef struct {
 	GLogLevelFlags	level;
 	MonoTraceMask	mask;
 } MonoLogLevelEntry;
 
-static GLogLevelFlags current_level		= G_LOG_LEVEL_ERROR;
-static MonoTraceMask current_mask		= MONO_TRACE_ALL;
+GLogLevelFlags mono_internal_current_level	= INT_MAX;
+MonoTraceMask  mono_internal_current_mask	= MONO_TRACE_ALL;
+gboolean mono_trace_log_header			= FALSE;
 
-static const char	*mono_log_domain	= "Mono";
 static GQueue		*level_stack		= NULL;
+static const char	*mono_log_domain	= "Mono";
 static MonoPrintCallback print_callback, printerr_callback;
 
+static MonoLogCallParm logCallback = {
+	.opener = NULL,
+	.writer = NULL,
+	.closer = NULL,
+	.header = FALSE
+};
+
+typedef struct {
+   MonoLogCallback legacy_callback;
+   gpointer user_data;
+} UserSuppliedLoggerUserData;
+
 /**
  * mono_trace_init:
  *
  * Initializes the mono tracer.
  */
-static void 
+void 
 mono_trace_init (void)
 {
 	if(level_stack == NULL) {
+		mono_internal_current_level = G_LOG_LEVEL_ERROR;
 		level_stack = g_queue_new();
 
 		mono_trace_set_mask_string(g_getenv("MONO_LOG_MASK"));
 		mono_trace_set_level_string(g_getenv("MONO_LOG_LEVEL"));
+		mono_trace_set_logheader_string(g_getenv("MONO_LOG_HEADER"));
+		mono_trace_set_logdest_string(g_getenv("MONO_LOG_DEST"));
 	}
 }
 
@@ -47,34 +63,12 @@ mono_trace_cleanup (void)
 			g_free (g_queue_pop_head (level_stack));
 		}
 
+		logCallback.closer();
 		g_queue_free (level_stack);
 		level_stack = NULL;
 	}
 }
 
-/**
- * mono_trace:
- *
- *	@level: Verbose level of the specified message
- *	@mask: Type of the specified message
- *
- * Traces a new message, depending on the current logging level
- * and trace mask.
- */
-void
-mono_trace(GLogLevelFlags level, MonoTraceMask mask, const char *format, ...) 
-{
-	if(level_stack == NULL)
-		mono_trace_init();
-
-	if(level <= current_level && mask & current_mask) {
-		va_list args;
-		va_start (args, format);
-		g_logv (mono_log_domain, level, format, args);
-		va_end (args);
-	}
-}
-
 /**
  * mono_tracev:
  *
@@ -85,13 +79,21 @@ mono_trace(GLogLevelFlags level, MonoTraceMask mask, const char *format, ...)
  * and trace mask.
  */
 void 
-mono_tracev (GLogLevelFlags level, MonoTraceMask mask, const char *format, va_list args)
+mono_tracev_inner (GLogLevelFlags level, MonoTraceMask mask, const char *format, va_list args)
 {
-	if (level_stack == NULL)
+	char *log_message;
+	if (level_stack == NULL) {
 		mono_trace_init ();
+		if(level > mono_internal_current_level || !(mask & mono_internal_current_mask))
+			return;
+	}
 
-	if(level <= current_level && mask & current_mask)
-		g_logv (mono_log_domain, level, format, args);
+	g_assert (logCallback.opener); // mono_trace_init should have provided us with one!
+
+	if (g_vasprintf (&log_message, format, args) < 0)
+		return;
+	logCallback.writer (mono_log_domain, level, logCallback.header, log_message);
+	g_free (log_message);
 }
 
 /**
@@ -109,7 +111,7 @@ mono_trace_set_level (GLogLevelFlags level)
 	if(level_stack == NULL)
 		mono_trace_init();
 
-	current_level = level;
+	mono_internal_current_level = level;
 }
 
 /**
@@ -127,7 +129,66 @@ mono_trace_set_mask (MonoTraceMask mask)
 	if(level_stack == NULL)
 		mono_trace_init();
 
-	current_mask	= mask;
+	mono_internal_current_mask = mask;
+}
+
+/**
+ * mono_trace_set_logdest:
+ *
+ *	@dest: Destination for logging
+ *
+ * Sets the current logging destination. This can be a file or, if supported,
+ * syslog.
+ */
+void 
+mono_trace_set_logdest_string (const char *dest)
+{
+	MonoLogCallParm logger;
+
+	if(level_stack == NULL)
+		mono_trace_init();
+
+#if PLATFORM_ANDROID
+	logger.opener = mono_log_open_logcat;
+	logger.writer = mono_log_write_logcat;
+	logger.closer = mono_log_close_logcat;
+	logger.dest   = (char*) dest;
+#elif defined (HOST_IOS)
+	logger.opener = mono_log_open_asl;
+	logger.writer = mono_log_write_asl;
+	logger.closer = mono_log_close_asl;
+	logger.dest   = (char*) dest;
+#else
+	if ((dest == NULL) || (strcmp("syslog", dest) != 0)) {
+		logger.opener = mono_log_open_logfile;
+		logger.writer = mono_log_write_logfile;
+		logger.closer = mono_log_close_logfile;
+		logger.dest   = (char *) dest;
+	} else {
+		logger.opener = mono_log_open_syslog;
+		logger.writer = mono_log_write_syslog;
+		logger.closer = mono_log_close_syslog;
+		logger.dest   = (char *) dest;
+	}
+#endif
+	mono_trace_set_log_handler_internal(&logger, NULL);
+}
+
+/**
+ * mono_trace_set_logheader:
+ *
+ *	@head: Whether we want pid/date/time header on log messages
+ *
+ * Sets the current logging header option.
+ */
+void 
+mono_trace_set_logheader_string(const char *head)
+{
+	if (head == NULL) {
+		mono_trace_log_header = FALSE;
+	} else {
+		mono_trace_log_header = TRUE;
+	}
 }
 
 /**
@@ -146,15 +207,15 @@ mono_trace_push (GLogLevelFlags level, MonoTraceMask mask)
 		g_error("%s: cannot use mono_trace_push without calling mono_trace_init first.", __func__);
 	else {
 		MonoLogLevelEntry *entry = (MonoLogLevelEntry *) g_malloc(sizeof(MonoLogLevelEntry));
-		entry->level	= current_level;
-		entry->mask		= current_mask;
+		entry->level	= mono_internal_current_level;
+		entry->mask		= mono_internal_current_mask;
 
 		g_queue_push_head (level_stack, (gpointer)entry);
 
 		/* Set the new level and mask
 		 */
-		current_level = level;
-		current_mask  = mask;
+		mono_internal_current_level = level;
+		mono_internal_current_mask  = mask;
 	}
 }
 
@@ -174,8 +235,8 @@ mono_trace_pop (void)
 
 			/*	Restore previous level and mask
 			 */
-			current_level = entry->level;
-			current_mask  = entry->mask;
+			mono_internal_current_level = entry->level;
+			mono_internal_current_mask  = entry->mask;
 
 			g_free (entry);
 		}
@@ -213,10 +274,11 @@ mono_trace_set_mask_string (const char *value)
 	const char *tok;
 	guint32 flags = 0;
 
-	const char *valid_flags[] = {"asm", "type", "dll", "gc", "cfg", "aot", "security", "threadpool", "all", NULL};
+	const char *valid_flags[] = {"asm", "type", "dll", "gc", "cfg", "aot", "security", "threadpool", "io-threadpool", "io-layer", "w32handle", "all", NULL};
 	const MonoTraceMask	valid_masks[] = {MONO_TRACE_ASSEMBLY, MONO_TRACE_TYPE, MONO_TRACE_DLLIMPORT,
 						 MONO_TRACE_GC, MONO_TRACE_CONFIG, MONO_TRACE_AOT, MONO_TRACE_SECURITY,
-						 MONO_TRACE_THREADPOOL, MONO_TRACE_ALL };
+						 MONO_TRACE_THREADPOOL, MONO_TRACE_IO_THREADPOOL, MONO_TRACE_IO_LAYER,
+						 MONO_TRACE_W32HANDLE, MONO_TRACE_ALL };
 
 	if(!value)
 		return;
@@ -253,33 +315,123 @@ mono_trace_set_mask_string (const char *value)
 gboolean
 mono_trace_is_traced (GLogLevelFlags level, MonoTraceMask mask)
 {
-	return (level <= current_level && mask & current_mask);
-}
-
-static MonoLogCallback log_callback;
-
-static const char*
-log_level_get_name (GLogLevelFlags log_level)
-{
-	switch (log_level & G_LOG_LEVEL_MASK) {
-	case G_LOG_LEVEL_ERROR: return "error";
-	case G_LOG_LEVEL_CRITICAL: return "critical";
-	case G_LOG_LEVEL_WARNING: return "warning";
-	case G_LOG_LEVEL_MESSAGE: return "message";
-	case G_LOG_LEVEL_INFO: return "info";
-	case G_LOG_LEVEL_DEBUG: return "debug";
-	default: return "unknown";
-	}
-}
-
-static void
-log_adapter (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
-{
-	log_callback (log_domain, log_level_get_name (log_level), message, log_level & G_LOG_LEVEL_ERROR, user_data);
+	return (level <= mono_internal_current_level && mask & mono_internal_current_mask);
 }
 
 /**
- * mono_trace_set_log_handler:
+ * log_level_get_name
+ * @log_level severity level
+ *
+ * Convert log level into a string for legacy log handlers
+ */
+static const char *
+log_level_get_name (GLogLevelFlags log_level)
+{
+        switch (log_level & G_LOG_LEVEL_MASK) {
+        case G_LOG_LEVEL_ERROR: return "error";
+        case G_LOG_LEVEL_CRITICAL: return "critical";
+        case G_LOG_LEVEL_WARNING: return "warning";
+        case G_LOG_LEVEL_MESSAGE: return "message";
+        case G_LOG_LEVEL_INFO: return "info";
+        case G_LOG_LEVEL_DEBUG: return "debug";
+        default: return "unknown";
+        }
+}
+
+/**
+ * callback_adapter
+ * 
+ *  @log_domain Message prefix
+ *  @log_level Severity
+ *  @message Message to be written
+ *  @fatal Fatal flag - write then abort
+ *  @user_data Argument passed to @callback
+ *
+ * This adapts the old callback writer exposed by MonoCallback to the newer method of
+ * logging. We ignore the header request as legacy handlers never had headers.
+ */
+static void
+callback_adapter (const char *domain, GLogLevelFlags level, mono_bool fatal, const char *message)
+{
+	UserSuppliedLoggerUserData *ll =logCallback.user_data;
+
+	ll->legacy_callback (domain, log_level_get_name(level), message, fatal, ll->user_data);
+}
+
+static void
+eglib_log_adapter (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
+{
+	UserSuppliedLoggerUserData *ll = logCallback.user_data;
+
+	ll->legacy_callback (log_domain, log_level_get_name (log_level), message, log_level & G_LOG_LEVEL_ERROR, ll->user_data);
+}
+
+/**
+ * legacy_opener
+ *
+ * Dummy routine for older style loggers
+ */
+static void
+legacy_opener(const char *path, void *user_data)
+{
+  /* nothing to do */
+}
+
+/**
+ * legacy_closer
+ *
+ * Cleanup routine for older style loggers
+ */
+static void
+legacy_closer(void)
+{
+	if (logCallback.user_data != NULL) {
+		g_free (logCallback.user_data); /* This is a UserSuppliedLoggerUserData struct */
+		logCallback.opener = NULL;	
+		logCallback.writer = NULL;
+		logCallback.closer = NULL;
+		logCallback.user_data = NULL;
+		logCallback.header = FALSE;
+	}
+}
+
+/**
+ *   mono_trace_set_log_handler:
+ *  
+ *  @callback The callback that will replace the default logging handler
+ *  @user_data Argument passed to @callback
+ * 
+ * The log handler replaces the default runtime logger. All logging requests with be routed to it.
+ * If the fatal argument in the callback is true, the callback must abort the current process. The runtime expects that
+ * execution will not resume after a fatal error.
+ */
+void
+mono_trace_set_log_handler (MonoLogCallback callback, void *user_data)
+{
+	g_assert (callback);
+
+	if (logCallback.closer != NULL)
+		logCallback.closer();
+	UserSuppliedLoggerUserData *ll = g_malloc (sizeof (UserSuppliedLoggerUserData));
+	ll->legacy_callback = callback;
+	ll->user_data = user_data;
+	logCallback.opener = legacy_opener;
+	logCallback.writer = callback_adapter;
+	logCallback.closer = legacy_closer;
+	logCallback.user_data = ll;
+	logCallback.dest = NULL;
+
+	g_log_set_default_handler (eglib_log_adapter, user_data);
+}
+
+static void
+structured_log_adapter (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
+{
+	logCallback.writer (log_domain, log_level, logCallback.header, message);
+}
+
+/**
+ * mono_trace_set_log_handler_internal:
  *
  *  @callback The callback that will replace the default logging handler
  *  @user_data Argument passed to @callback
@@ -289,11 +441,19 @@ log_adapter (const gchar *log_domain, GLogLevelFlags log_level, const gchar *mes
  * execution will not resume after a fatal error.
  */
 void
-mono_trace_set_log_handler (MonoLogCallback callback, void *user_data)
+mono_trace_set_log_handler_internal (MonoLogCallParm *callback, void *user_data)
 {
 	g_assert (callback);
-	log_callback = callback;
-	g_log_set_default_handler (log_adapter, user_data);
+	if (logCallback.closer != NULL)
+		logCallback.closer();
+	logCallback.opener = callback->opener;
+	logCallback.writer = callback->writer;
+	logCallback.closer = callback->closer;
+	logCallback.header = mono_trace_log_header;
+	logCallback.dest   = callback->dest;
+	logCallback.opener (logCallback.dest, user_data);
+
+	g_log_set_default_handler (structured_log_adapter, user_data);
 }
 
 static void
