@@ -5,6 +5,7 @@
  *   Mono Team (mono-list@lists.ximian.com)
  *
  * Copyright 2001-2008 Novell, Inc.
+ * Licensed under the MIT license. See LICENSE file in the project root for full license information.
  */
 
 #include "config.h"
@@ -483,7 +484,7 @@ mono_pagesize (void)
 void*
 mono_valloc (void *addr, size_t length, int flags)
 {
-	return malloc (length);
+	return g_malloc (length);
 }
 
 void*
@@ -497,7 +498,7 @@ mono_valloc_aligned (size_t length, size_t alignment, int flags)
 int
 mono_vfree (void *addr, size_t length)
 {
-	free (addr);
+	g_free (addr);
 	return 0;
 }
 
